"""Agent card panel component for displaying agent metadata and capabilities."""

import json
from typing import Any

from a2a.types import AgentCard
from rich.syntax import Syntax
from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Container, VerticalScroll
from textual.widgets import Static

from a2a_handler.common import get_logger

logger = get_logger(__name__)

TEXTUAL_TO_SYNTAX_THEME_MAP: dict[str, str] = {
    "gruvbox": "gruvbox-dark",
    "nord": "nord",
    "textual-light": "default",
    "solarized-light": "solarized-light",
    "dracula": "dracula",
}


class AgentCardPanel(Container):
    """Panel displaying agent card information with tabs."""

    BINDINGS = [
        Binding("j", "scroll_down", "↓ Scroll", show=True, key_display="j/↓"),
        Binding("k", "scroll_up", "↑ Scroll", show=True, key_display="k/↑"),
        Binding("down", "scroll_down", "Scroll Down", show=False),
        Binding("up", "scroll_up", "Scroll Up", show=False),
        Binding("ctrl+d", "scroll_half_down", "½ Page ↓", show=True),
        Binding("ctrl+u", "scroll_half_up", "½ Page ↑", show=True),
    ]

    can_focus = True

    def check_action(self, action: str, parameters: tuple[object, ...]) -> bool | None:
        """Hide scroll actions when no agent card is loaded."""
        if action in ("scroll_down", "scroll_up", "scroll_half_down", "scroll_half_up"):
            return self._current_agent_card is not None
        return True

    def __init__(self, **kwargs: Any) -> None:
        super().__init__(**kwargs)
        self._current_agent_card: AgentCard | None = None

    def compose(self) -> ComposeResult:
        yield Static("Connect to an A2A server", id="placeholder")
        yield VerticalScroll(
            Static("", id="agent-raw"),
            id="raw-scroll",
        )

    def on_mount(self) -> None:
        for widget in self.query("VerticalScroll"):
            widget.can_focus = False
        self._show_placeholder()
        logger.debug("Agent card panel mounted")

    def _show_placeholder(self) -> None:
        """Show the hatch placeholder, hide the raw scroll content."""
        placeholder = self.query_one("#placeholder", Static)
        raw_scroll = self.query_one("#raw-scroll", VerticalScroll)
        placeholder.display = True
        raw_scroll.display = False

    def _show_content(self) -> None:
        """Show the raw scroll content, hide the placeholder."""
        placeholder = self.query_one("#placeholder", Static)
        raw_scroll = self.query_one("#raw-scroll", VerticalScroll)
        placeholder.display = False
        raw_scroll.display = True

    def _get_syntax_theme_for_current_app_theme(self) -> str | None:
        """Get the Rich Syntax theme name for the current app theme."""
        current_theme = self.app.theme or ""
        return TEXTUAL_TO_SYNTAX_THEME_MAP.get(current_theme)

    def update_card(self, agent_card: AgentCard | None) -> None:
        """Update the displayed agent card."""
        self._current_agent_card = agent_card
        self.refresh_bindings()

        raw_view_widget = self.query_one("#agent-raw", Static)

        if agent_card is None:
            logger.debug("Clearing agent card display")
            raw_view_widget.update("")
            self._show_placeholder()
        else:
            logger.info("Displaying agent card for: %s", agent_card.name)
            json_content = json.dumps(agent_card.model_dump(), indent=2, default=str)
            syntax_theme = self._get_syntax_theme_for_current_app_theme()
            if syntax_theme:
                raw_view_widget.update(Syntax(json_content, "json", theme=syntax_theme))
            else:
                raw_view_widget.update(json_content)
            self._show_content()

    def refresh_theme(self) -> None:
        """Refresh the raw view syntax highlighting for theme changes."""
        if self._current_agent_card is None:
            return

        logger.debug("Refreshing syntax theme for agent card raw view")
        json_content = json.dumps(
            self._current_agent_card.model_dump(), indent=2, default=str
        )
        syntax_theme = self._get_syntax_theme_for_current_app_theme()
        raw_widget = self.query_one("#agent-raw", Static)
        if syntax_theme:
            raw_widget.update(Syntax(json_content, "json", theme=syntax_theme))
        else:
            raw_widget.update(json_content)

    def action_scroll_down(self) -> None:
        """Scroll down in the scroll container."""
        scroll_container = self.query_one("#raw-scroll", VerticalScroll)
        scroll_container.scroll_down()

    def action_scroll_up(self) -> None:
        """Scroll up in the scroll container."""
        scroll_container = self.query_one("#raw-scroll", VerticalScroll)
        scroll_container.scroll_up()

    def action_scroll_half_down(self) -> None:
        """Scroll down half a page."""
        scroll_container = self.query_one("#raw-scroll", VerticalScroll)
        scroll_container.scroll_relative(y=scroll_container.size.height // 2)

    def action_scroll_half_up(self) -> None:
        """Scroll up half a page."""
        scroll_container = self.query_one("#raw-scroll", VerticalScroll)
        scroll_container.scroll_relative(y=-(scroll_container.size.height // 2))



"""Shared utilities for CLI commands."""

import httpx
import rich_click as click
from a2a.client.errors import (
    A2AClientError,
    A2AClientHTTPError,
    A2AClientTimeoutError,
)

from a2a_handler.common import Output, get_logger

TIMEOUT = 120
log = get_logger(__name__)


def build_http_client(timeout: int = TIMEOUT) -> httpx.AsyncClient:
    """Build an HTTP client with the specified timeout."""
    return httpx.AsyncClient(timeout=timeout)


def handle_client_error(e: Exception, agent_url: str, output: Output | None) -> None:
    """Handle A2A client errors with appropriate messages."""
    message = ""
    if isinstance(e, A2AClientTimeoutError):
        log.error("Request to %s timed out", agent_url)
        message = "Request timed out"
    elif isinstance(e, A2AClientHTTPError):
        log.error("A2A client error: %s", e)
        message = (
            f"Connection failed: Is the server running at {agent_url}?"
            if "connection" in str(e).lower()
            else str(e)
        )
    elif isinstance(e, A2AClientError):
        log.error("A2A client error: %s", e)
        message = str(e)
    elif isinstance(e, httpx.ConnectError):
        log.error("Connection refused to %s", agent_url)
        message = f"Connection refused: Is the server running at {agent_url}?"
    elif isinstance(e, httpx.TimeoutException):
        log.error("Request to %s timed out", agent_url)
        message = "Request timed out"
    elif isinstance(e, httpx.HTTPStatusError):
        log.error("HTTP error %d from %s", e.response.status_code, agent_url)
        message = f"HTTP {e.response.status_code} - {e.response.text}"
    else:
        log.exception("Failed request to %s", agent_url)
        message = str(e)

    if output:
        output.error(message)
    else:
        click.echo(f"Error: {message}", err=True)



"""Server commands for running local A2A servers."""

from typing import Optional

import rich_click as click

from a2a_handler.common import get_logger
from a2a_handler.server import run_server
from a2a_handler.webhook import run_webhook_server

log = get_logger(__name__)


@click.group()
def server() -> None:
    """Run local servers."""
    pass


@server.command("agent")
@click.option("--host", default="0.0.0.0", help="Host to bind to", show_default=True)
@click.option("--port", default=8000, help="Port to bind to", show_default=True)
@click.option("--auth/--no-auth", default=False, help="Require API key authentication")
@click.option(
    "--api-key",
    default=None,
    help="Specific API key to use (auto-generated if not set)",
)
@click.option(
    "--model",
    "-m",
    default=None,
    help="Model to use (e.g., 'llama3.2:1b', 'qwen3', 'gemini-2.0-flash')",
)
def server_agent(
    host: str,
    port: int,
    auth: bool,
    api_key: Optional[str],
    model: Optional[str],
) -> None:
    """Start a local A2A agent server."""
    log.info("Starting A2A server on %s:%d", host, port)
    run_server(
        host=host,
        port=port,
        require_auth=auth,
        api_key=api_key,
        model=model,
    )


@server.command("push")
@click.option("--host", default="127.0.0.1", help="Host to bind to", show_default=True)
@click.option("--port", default=9000, help="Port to bind to", show_default=True)
def server_push(host: str, port: int) -> None:
    """Start a local webhook server for receiving push notifications."""
    log.info("Starting webhook server on %s:%d", host, port)
    run_webhook_server(host, port)



"""Simple output formatting system for CLI.

Provides styled console output with ANSI colors.
"""

from __future__ import annotations

import json as json_module
import sys
from typing import Any, TextIO

TERMINAL_STATES = {"completed", "failed", "canceled", "rejected"}
SUCCESS_STATES = {"completed"}
ERROR_STATES = {"failed", "rejected"}
WARNING_STATES = {"canceled"}

# Basic ANSI color codes
RESET = "\033[0m"
BOLD = "\033[1m"
DIM = "\033[2m"
RED = "\033[31m"
GREEN = "\033[32m"
YELLOW = "\033[33m"
CYAN = "\033[36m"


def _supports_color(stream: TextIO) -> bool:
    """Check if the stream supports ANSI colors."""
    if not hasattr(stream, "isatty"):
        return False
    if not stream.isatty():
        return False
    return True


class Output:
    """Manages styled console output.

    Provides a unified interface for outputting text, fields, JSON, and
    markdown with automatic color formatting when supported.
    """

    def __init__(self) -> None:
        self._use_color = _supports_color(sys.stdout)

    def _style(self, text: str, *codes: str) -> str:
        """Apply ANSI codes if color is enabled."""
        if not self._use_color or not codes:
            return text
        return "".join(codes) + text + RESET

    def _print(self, text: str) -> None:
        """Print text to stdout."""
        print(text)

    def line(self, text: str, style: str | None = None) -> None:
        """Print a line of text with optional style."""
        if style and self._use_color:
            code = {
                "green": GREEN,
                "red": RED,
                "yellow": YELLOW,
                "cyan": CYAN,
                "dim": DIM,
                "bold": BOLD,
            }.get(style, "")
            text = self._style(text, code)
        self._print(text)

    def field(
        self,
        name: str,
        value: Any,
        dim_value: bool = False,
        value_style: str | None = None,
    ) -> None:
        """Print a field as 'Name: value' with formatting."""
        value_str = str(value) if value is not None else "none"
        name_part = self._style(f"{name}:", BOLD) if self._use_color else f"{name}:"

        if self._use_color:
            if value_style:
                code = {"green": GREEN, "red": RED, "cyan": CYAN}.get(value_style, "")
                value_part = self._style(value_str, code)
            elif dim_value:
                value_part = self._style(value_str, DIM)
            else:
                value_part = value_str
        else:
            value_part = value_str

        self._print(f"{name_part} {value_part}")

    def header(self, text: str) -> None:
        """Print a section header."""
        styled = self._style(text, BOLD) if self._use_color else text
        self._print(f"\n{styled}")

    def subheader(self, text: str) -> None:
        """Print a subheader (less prominent than header)."""
        styled = self._style(text, BOLD, CYAN) if self._use_color else text
        self._print(styled)

    def blank(self) -> None:
        """Print a blank line."""
        self._print("")

    def state(self, name: str, state: str) -> None:
        """Print a state field with appropriate coloring."""
        lower = state.lower()
        if lower in SUCCESS_STATES:
            style = "green"
        elif lower in ERROR_STATES:
            style = "red"
        elif lower in WARNING_STATES:
            style = "yellow"
        else:
            style = "cyan"

        name_part = self._style(f"{name}:", BOLD) if self._use_color else f"{name}:"
        code = {"green": GREEN, "red": RED, "yellow": YELLOW, "cyan": CYAN}.get(
            style, ""
        )
        value_part = self._style(state, code) if self._use_color else state
        self._print(f"{name_part} {value_part}")

    def success(self, text: str) -> None:
        """Print a success message."""
        self.line(text, "green")

    def error(self, text: str) -> None:
        """Print an error message."""
        styled = self._style(text, RED, BOLD) if self._use_color else text
        self._print(styled)

    def warning(self, text: str) -> None:
        """Print a warning message."""
        self.line(text, "yellow")

    def dim(self, text: str) -> None:
        """Print dimmed/muted text."""
        self.line(text, "dim")

    def json(self, data: Any) -> None:
        """Print JSON data."""
        json_str = json_module.dumps(data, indent=2, default=str)
        self._print(json_str)

    def markdown(self, text: str) -> None:
        """Print markdown content (as plain text)."""
        self._print(text)

    def list_item(self, text: str, bullet: str = "•") -> None:
        """Print a list item with bullet."""
        self._print(f"  {bullet} {text}")



"""A2A application setup and middleware."""

import secrets
from collections.abc import Awaitable, Callable

import httpx
from a2a.server.apps import A2AStarletteApplication
from a2a.server.request_handlers import DefaultRequestHandler
from a2a.server.tasks import (
    BasePushNotificationSender,
    InMemoryPushNotificationConfigStore,
    InMemoryTaskStore,
)
from a2a.types import AgentCard
from google.adk.a2a.executor.a2a_agent_executor import A2aAgentExecutor
from google.adk.agents.llm_agent import Agent
from google.adk.artifacts.in_memory_artifact_service import InMemoryArtifactService
from google.adk.auth.credential_service.in_memory_credential_service import (
    InMemoryCredentialService,
)
from google.adk.memory.in_memory_memory_service import InMemoryMemoryService
from google.adk.runners import Runner
from google.adk.sessions.in_memory_session_service import InMemorySessionService
from starlette.applications import Starlette
from starlette.middleware import Middleware
from starlette.middleware.base import BaseHTTPMiddleware
from starlette.requests import Request
from starlette.responses import JSONResponse, Response

from a2a_handler.common import get_logger

logger = get_logger(__name__)

DEFAULT_HTTP_TIMEOUT_SECONDS = 30


def generate_api_key() -> str:
    """Generate a secure random API key.

    Returns:
        A URL-safe random string suitable for use as an API key
    """
    return secrets.token_urlsafe(32)


class APIKeyAuthMiddleware(BaseHTTPMiddleware):
    """Middleware to enforce API key authentication on A2A endpoints."""

    OPEN_PATHS = {
        "/.well-known/agent-card.json",
        "/health",
    }

    def __init__(self, app: Starlette, api_key: str) -> None:
        super().__init__(app)
        self.api_key = api_key

    async def dispatch(self, request: Request, call_next: Callable) -> Response:
        if request.url.path in self.OPEN_PATHS:
            return await call_next(request)

        if request.method == "GET" and request.url.path == "/":
            return await call_next(request)

        auth_header = request.headers.get("Authorization")
        api_key_header = request.headers.get("X-API-Key")

        authenticated = False

        if api_key_header and api_key_header == self.api_key:
            authenticated = True
        elif auth_header:
            if auth_header.startswith("Bearer "):
                token = auth_header[7:]
                if token == self.api_key:
                    authenticated = True
            elif auth_header.startswith("ApiKey "):
                token = auth_header[7:]
                if token == self.api_key:
                    authenticated = True

        if not authenticated:
            return JSONResponse(
                status_code=401,
                content={
                    "jsonrpc": "2.0",
                    "error": {
                        "code": -32000,
                        "message": "Unauthorized: Invalid or missing API key",
                    },
                    "id": None,
                },
                headers={"WWW-Authenticate": 'ApiKey realm="Handler Server"'},
            )

        return await call_next(request)


def create_runner_factory(agent: Agent) -> Callable[[], Awaitable[Runner]]:
    """Create a factory function that builds a Runner for the agent.

    Args:
        agent: The ADK agent to wrap

    Returns:
        A callable that creates a Runner instance
    """

    async def create_runner() -> Runner:
        return Runner(
            app_name=agent.name or "handler_agent",
            agent=agent,
            artifact_service=InMemoryArtifactService(),
            session_service=InMemorySessionService(),
            memory_service=InMemoryMemoryService(),
            credential_service=InMemoryCredentialService(),
        )

    return create_runner


def create_a2a_application(
    agent: Agent,
    agent_card: AgentCard,
    api_key: str | None = None,
) -> Starlette:
    """Create a Starlette A2A application with full push notification support.

    This is a custom implementation that replaces google-adk's to_a2a() to add
    push notification support. The to_a2a() function doesn't pass push_config_store
    or push_sender to DefaultRequestHandler, causing push notification operations
    to fail with "UnsupportedOperationError".

    Args:
        agent: The ADK agent
        agent_card: Pre-configured agent card
        api_key: Optional API key for authentication

    Returns:
        Configured Starlette application
    """
    task_store = InMemoryTaskStore()
    push_notification_config_store = InMemoryPushNotificationConfigStore()
    http_client = httpx.AsyncClient(timeout=DEFAULT_HTTP_TIMEOUT_SECONDS)
    push_notification_sender = BasePushNotificationSender(
        http_client, push_notification_config_store
    )

    agent_executor = A2aAgentExecutor(
        runner=create_runner_factory(agent),
    )

    request_handler = DefaultRequestHandler(
        agent_executor=agent_executor,
        task_store=task_store,
        push_config_store=push_notification_config_store,
        push_sender=push_notification_sender,
    )

    middleware: list[Middleware] = []
    if api_key:
        middleware.append(
            Middleware(APIKeyAuthMiddleware, api_key=api_key)  # type: ignore[arg-type]
        )

    application = Starlette(middleware=middleware)

    async def setup_a2a_routes() -> None:
        a2a_starlette_app = A2AStarletteApplication(
            agent_card=agent_card,
            http_handler=request_handler,
        )
        a2a_starlette_app.add_routes_to_app(application)
        logger.info("A2A routes configured with push notification support")

    async def cleanup_http_client() -> None:
        await http_client.aclose()
        logger.info("HTTP client closed")

    application.add_event_handler("startup", setup_a2a_routes)
    application.add_event_handler("shutdown", cleanup_http_client)

    return application



"""Command-line interface for the Handler A2A protocol client.

Provides commands for interacting with A2A agents:
- message send/stream: Send messages to agents
- task get/cancel/resubscribe: Manage tasks
- task notification set: Configure push notifications
- card get/validate: Agent card operations
- server agent/push: Run local servers
- session list/show/clear: Manage saved sessions
"""

import truststore

truststore.inject_into_ssl()

import logging

logging.getLogger().setLevel(logging.WARNING)

import rich_click as click

from a2a_handler import __version__
from a2a_handler.common import get_logger, setup_logging
from a2a_handler.tui import HandlerTUI

from . import _config  # noqa: F401 - configures rich-click on import
from .auth import auth
from .card import card
from .mcp import mcp
from .message import message
from .server import server
from .session import session
from .task import task

log = get_logger(__name__)


@click.group()
@click.option("--verbose", "-v", is_flag=True, help="Enable verbose logging")
@click.option("--debug", "-d", is_flag=True, help="Enable debug logging")
@click.pass_context
def cli(ctx: click.Context, verbose: bool, debug: bool) -> None:
    """Handler - A2A protocol client CLI."""
    ctx.ensure_object(dict)

    if debug:
        setup_logging(level="DEBUG")
    elif verbose:
        setup_logging(level="INFO")
    else:
        setup_logging(level="ERROR")


cli.add_command(message)
cli.add_command(task)
cli.add_command(card)
cli.add_command(server)
cli.add_command(session)
cli.add_command(auth)
cli.add_command(mcp)


@cli.command()
def version() -> None:
    """Display the current version."""
    click.echo(__version__)


@cli.command()
def tui() -> None:
    """Launch the interactive terminal interface."""
    log.info("Launching TUI")
    logging.getLogger().handlers = []
    app = HandlerTUI()
    app.run()


@cli.command()
@click.option("--host", default="localhost", help="Host to bind to", show_default=True)
@click.option("--port", "-p", default=8001, help="Port to bind to", show_default=True)
def web(host: str, port: int) -> None:
    """Serve the TUI as a web application."""
    from textual_serve.server import Server

    log.info("Starting web server on %s:%d", host, port)
    server = Server(
        command="handler tui",
        host=host,
        port=port,
        title="Handler",
    )
    server.serve()


def main() -> None:
    """Main entry point for the CLI."""
    cli()


if __name__ == "__main__":
    main()



"""Authentication panel component for configuring agent auth."""

from __future__ import annotations

from textual.app import ComposeResult
from textual.containers import Vertical
from textual.widgets import Input, Label, RadioButton, RadioSet

from a2a_handler.auth import (
    AuthCredentials,
    AuthType,
    create_api_key_auth,
    create_bearer_auth,
)
from a2a_handler.common import get_logger

logger = get_logger(__name__)


class AuthPanel(Vertical):
    """Panel for configuring authentication credentials."""

    can_focus = False

    def compose(self) -> ComposeResult:
        yield Label("Authentication Type")
        with RadioSet(id="auth-type-selector"):
            yield RadioButton("None", id="auth-none", value=True)
            yield RadioButton("API Key", id="auth-api-key")
            yield RadioButton("Bearer Token", id="auth-bearer")

        with Vertical(id="api-key-fields", classes="auth-fields hidden"):
            yield Label("API Key")
            yield Input(placeholder="Enter API key", id="api-key-input", password=True)
            yield Label("Header Name")
            yield Input(placeholder="X-API-Key", id="api-key-header-input")

        with Vertical(id="bearer-fields", classes="auth-fields hidden"):
            yield Label("Bearer Token")
            yield Input(
                placeholder="Enter bearer token", id="bearer-token-input", password=True
            )

    def on_radio_set_changed(self, event: RadioSet.Changed) -> None:
        """Handle auth type selection changes."""
        api_key_fields = self.query_one("#api-key-fields", Vertical)
        bearer_fields = self.query_one("#bearer-fields", Vertical)

        api_key_fields.add_class("hidden")
        bearer_fields.add_class("hidden")

        if event.pressed.id == "auth-api-key":
            api_key_fields.remove_class("hidden")
            logger.debug("Auth type changed to API Key")
        elif event.pressed.id == "auth-bearer":
            bearer_fields.remove_class("hidden")
            logger.debug("Auth type changed to Bearer Token")
        else:
            logger.debug("Auth type changed to None")

    def get_credentials(self) -> AuthCredentials | None:
        """Get the configured authentication credentials.

        Returns:
            AuthCredentials if auth is configured, None otherwise.
        """
        radio_set = self.query_one("#auth-type-selector", RadioSet)
        pressed = radio_set.pressed_button

        if pressed is None or pressed.id == "auth-none":
            return None

        if pressed.id == "auth-api-key":
            api_key = self.query_one("#api-key-input", Input).value
            header_name = (
                self.query_one("#api-key-header-input", Input).value or "X-API-Key"
            )
            if api_key:
                return create_api_key_auth(api_key, header_name=header_name)

        elif pressed.id == "auth-bearer":
            token = self.query_one("#bearer-token-input", Input).value
            if token:
                return create_bearer_auth(token)

        return None

    def get_auth_type(self) -> AuthType | None:
        """Get the currently selected auth type."""
        radio_set = self.query_one("#auth-type-selector", RadioSet)
        pressed = radio_set.pressed_button

        if pressed is None or pressed.id == "auth-none":
            return None
        elif pressed.id == "auth-api-key":
            return AuthType.API_KEY
        elif pressed.id == "auth-bearer":
            return AuthType.BEARER
        return None



"""TUI component widgets for the Handler application."""

from .artifacts import ArtifactsPanel
from .auth import AuthPanel
from .card import AgentCardPanel
from .contact import ContactPanel
from .input import InputPanel
from .logs import LogsPanel
from .messages import Message, MessagesPanel, TabbedMessagesPanel
from .tasks import TasksPanel

__all__ = [
    "AgentCardPanel",
    "ArtifactsPanel",
    "AuthPanel",
    "ContactPanel",
    "InputPanel",
    "LogsPanel",
    "Message",
    "MessagesPanel",
    "TabbedMessagesPanel",
    "TasksPanel",
]



"""Session commands for managing saved session state."""

from typing import Optional

import rich_click as click

from a2a_handler.common import Output
from a2a_handler.session import clear_session, get_session, get_session_store


@click.group()
def session() -> None:
    """Manage saved session state."""
    pass


@session.command("list")
def session_list() -> None:
    """List all saved sessions."""
    output = Output()
    store = get_session_store()
    sessions = store.list_all()

    if not sessions:
        output.dim("No saved sessions")
        return

    output.header(f"Saved Sessions ({len(sessions)})")
    for s in sessions:
        output.blank()
        output.subheader(s.agent_url)
        if s.context_id:
            output.field("Context ID", s.context_id, dim_value=True)
        if s.task_id:
            output.field("Task ID", s.task_id, dim_value=True)


@session.command("show")
@click.argument("agent_url")
def session_show(agent_url: str) -> None:
    """Display session state for an agent."""
    output = Output()
    s = get_session(agent_url)
    output.header(f"Session for {agent_url}")
    output.field("Context ID", s.context_id or "none", dim_value=not s.context_id)
    output.field("Task ID", s.task_id or "none", dim_value=not s.task_id)


@session.command("clear")
@click.argument("agent_url", required=False)
@click.option("--all", "-a", "clear_all", is_flag=True, help="Clear all sessions")
def session_clear(agent_url: Optional[str], clear_all: bool) -> None:
    """Clear saved session state."""
    output = Output()
    if clear_all:
        clear_session()
        output.success("Cleared all sessions")
    elif agent_url:
        clear_session(agent_url)
        output.success(f"Cleared session for {agent_url}")
    else:
        output.warning("Provide AGENT_URL or use --all to clear sessions")



"""Main TUI application for Handler.

Provides the Textual-based terminal interface for agent interaction.
"""

import logging
import uuid
from collections.abc import Iterable
from importlib.metadata import version
from typing import Any

import httpx
from a2a.types import AgentCard
from textual import on, work
from textual.app import App, ComposeResult, SystemCommand
from textual.binding import Binding
from textual.containers import Container, Vertical
from textual.logging import TextualHandler
from textual.screen import Screen
from textual.widgets import Button, Footer, Input

from a2a_handler.common import get_theme, install_tui_log_handler, save_theme
from a2a_handler.service import A2AService
from a2a_handler.tui.components import (
    AgentCardPanel,
    ContactPanel,
    InputPanel,
    TabbedMessagesPanel,
)

__version__ = version("a2a-handler")

logging.basicConfig(
    level="NOTSET",
    handlers=[TextualHandler()],
)
logger = logging.getLogger(__name__)

DEFAULT_HTTP_TIMEOUT_SECONDS = 120


def build_http_client(
    timeout_seconds: int = DEFAULT_HTTP_TIMEOUT_SECONDS,
) -> httpx.AsyncClient:
    """Build an HTTP client with the specified timeout."""
    return httpx.AsyncClient(timeout=timeout_seconds)


class HandlerTUI(App[Any]):
    """Handler - A2A Agent Management Interface."""

    CSS_PATH = "app.tcss"

    BINDINGS = [
        Binding("ctrl+q", "quit", "Quit", show=True),
        Binding("/", "command_palette", "Palette", show=True),
        Binding("ctrl+m", "toggle_maximize", "Maximize", show=True),
    ]

    def check_action(self, action: str, parameters: tuple[object, ...]) -> bool | None:
        """Show maximize binding only for maximizable panels."""
        if action == "toggle_maximize":
            focused = self.focused
            if focused is None:
                return False
            for panel in (
                self.query_one("#messages-container", TabbedMessagesPanel),
                self.query_one("#agent-card-container", AgentCardPanel),
            ):
                if focused is panel or panel in focused.ancestors:
                    return True
            return False
        return True

    def __init__(self, **kwargs: Any) -> None:
        super().__init__(**kwargs)
        self.current_agent_card: AgentCard | None = None
        self.http_client: httpx.AsyncClient | None = None
        self.current_context_id: str | None = None
        self.current_agent_url: str | None = None
        self._agent_service: A2AService | None = None
        self._is_maximized: bool = False

    def compose(self) -> ComposeResult:
        with Container(id="root-container"):
            with Vertical(id="left-pane"):
                yield ContactPanel(id="contact-container", classes="panel")
                yield AgentCardPanel(id="agent-card-container", classes="panel")

            with Vertical(id="right-pane"):
                yield TabbedMessagesPanel(id="messages-container", classes="panel")
                yield InputPanel(id="input-container", classes="panel")
        yield Footer(show_command_palette=False)

    async def on_mount(self) -> None:
        logger.info("TUI application starting")
        self.http_client = build_http_client()
        self.theme = get_theme()

        tui_log_handler = install_tui_log_handler(level=logging.DEBUG)
        tui_log_handler.set_callback(self._on_log_line)

        messages_panel = self.query_one("#messages-container", TabbedMessagesPanel)
        messages_panel.load_logs(tui_log_handler.get_lines())

        contact_panel = self.query_one("#contact-container", ContactPanel)
        contact_panel.set_version(__version__)

        messages_panel.add_system_message(
            "Welcome! Connect to an agent to start chatting."
        )

    def _on_log_line(self, line: str) -> None:
        """Callback for new log lines."""
        try:
            messages_panel = self.query_one("#messages-container", TabbedMessagesPanel)
            messages_panel.add_log(line)
        except Exception:
            pass

    def watch_theme(self, new_theme: str) -> None:
        """Called when the app theme changes."""
        logger.debug("Theme changed to: %s", new_theme)
        save_theme(new_theme)
        agent_card_panel = self.query_one("#agent-card-container", AgentCardPanel)
        agent_card_panel.refresh_theme()

    async def _connect_to_agent(self, agent_url: str) -> AgentCard:
        if not self.http_client:
            raise RuntimeError("HTTP client not initialized")

        logger.info("Connecting to agent at %s", agent_url)
        self._agent_service = A2AService(self.http_client, agent_url)
        return await self._agent_service.get_card()

    def _update_ui_for_connected_state(self, agent_card: AgentCard) -> None:
        agent_card_panel = self.query_one("#agent-card-container", AgentCardPanel)
        agent_card_panel.update_card(agent_card)

        messages_panel = self.query_one("#messages-container", TabbedMessagesPanel)
        messages_panel.update_message_count()

    @on(Button.Pressed, "#connect-btn")
    async def handle_connect_button(self) -> None:
        contact_panel = self.query_one("#contact-container", ContactPanel)
        agent_url = contact_panel.get_url()

        if not agent_url:
            logger.warning("Connect attempted with empty URL")
            messages_panel = self.query_one("#messages-container", TabbedMessagesPanel)
            messages_panel.add_system_message("Please enter an agent URL")
            return

        messages_panel = self.query_one("#messages-container", TabbedMessagesPanel)
        messages_panel.add_system_message(f"Connecting to {agent_url}...")

        try:
            agent_card = await self._connect_to_agent(agent_url)

            self.current_agent_card = agent_card
            self.current_agent_url = agent_url
            self.current_context_id = str(uuid.uuid4())

            logger.info("Successfully connected to %s", agent_card.name)

            self._update_ui_for_connected_state(agent_card)
            messages_panel.add_system_message(f"Connected to {agent_card.name}")

            agent_card_panel = self.query_one("#agent-card-container", AgentCardPanel)
            agent_card_panel.focus()

        except Exception as error:
            logger.error("Connection failed: %s", error, exc_info=True)
            messages_panel.add_system_message(f"Connection failed: {error!s}")
            agent_card_panel = self.query_one("#agent-card-container", AgentCardPanel)
            agent_card_panel.update_card(None)

    @on(Input.Submitted, "#message-input")
    def handle_message_submit(self) -> None:
        if self.current_agent_url:
            self._send_message()
        else:
            messages_panel = self.query_one("#messages-container", TabbedMessagesPanel)
            messages_panel.add_system_message("Not connected to an agent")

    @on(Button.Pressed, "#send-btn")
    def handle_send_button(self) -> None:
        if self.current_agent_url:
            self._send_message()
        else:
            messages_panel = self.query_one("#messages-container", TabbedMessagesPanel)
            messages_panel.add_system_message("Not connected to an agent")

    @work(exclusive=True)
    async def _send_message(self) -> None:
        if not self.current_agent_url or not self._agent_service:
            logger.warning("Attempted to send message without connection")
            messages_panel = self.query_one("#messages-container", TabbedMessagesPanel)
            messages_panel.add_system_message("Not connected to an agent")
            return

        input_panel = self.query_one("#input-container", InputPanel)
        message_text = input_panel.get_message()

        if not message_text:
            return

        messages_panel = self.query_one("#messages-container", TabbedMessagesPanel)
        messages_panel.add_message("user", message_text)

        try:
            logger.info("Sending message: %s", message_text[:50])

            credentials = messages_panel.get_auth_credentials()
            if credentials:
                self._agent_service.set_credentials(credentials)

            send_result = await self._agent_service.send(
                message_text,
                context_id=self.current_context_id,
            )

            if send_result.context_id:
                self.current_context_id = send_result.context_id

            logger.info(
                "Response received - task_id=%s, state=%s, has_text=%s, has_task=%s, has_message=%s",
                send_result.task_id,
                send_result.state,
                bool(send_result.text),
                send_result.task is not None,
                send_result.message is not None,
            )
            if send_result.task:
                logger.debug("Raw response: %s", send_result.task.model_dump())
            elif send_result.message:
                logger.debug("Raw response: %s", send_result.message.model_dump())

            messages_panel.add_agent_message(send_result)

            if send_result.task:
                messages_panel.update_task(send_result.task)

                if send_result.task.artifacts:
                    for artifact in send_result.task.artifacts:
                        messages_panel.update_artifact(
                            artifact,
                            send_result.task_id or "",
                            self.current_context_id or "",
                        )

        except Exception as error:
            logger.error("Error sending message: %s", error, exc_info=True)
            messages_panel.add_system_message(f"Error: {error!s}")

    def action_toggle_maximize(self) -> None:
        """Toggle maximize for the focused panel."""
        if self._is_maximized:
            self.screen.minimize()
            self._is_maximized = False
            return

        focused = self.focused
        if focused is None:
            return

        for panel in (
            self.query_one("#messages-container", TabbedMessagesPanel),
            self.query_one("#agent-card-container", AgentCardPanel),
        ):
            if focused is panel or panel in focused.ancestors:
                self.screen.maximize(panel)
                self._is_maximized = True
                return

    def get_system_commands(self, screen: Screen) -> Iterable[SystemCommand]:
        """Filter out maximize/minimize commands from the command palette."""
        for command in super().get_system_commands(screen):
            if command.title.lower() in ("maximize", "minimize"):
                continue
            yield command

    async def on_unmount(self) -> None:
        logger.info("Shutting down TUI application")
        if self.http_client:
            await self.http_client.aclose()


def main() -> None:
    """Entry point for the TUI application."""
    application = HandlerTUI()
    application.run()


if __name__ == "__main__":
    main()



"""Contact panel component for managing agent connections."""

import webbrowser
from typing import Any

from textual import on
from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Container, Horizontal, Vertical
from textual.widgets import Button, Input, Link, Static, TabbedContent, TabPane, Tabs

from a2a_handler.common import get_logger

logger = get_logger(__name__)

REPORT_BUG_URL = "https://github.com/alDuncanson/Handler/issues"
SPONSOR_URL = "https://github.com/sponsors/alDuncanson"
DISCUSS_URL = "https://github.com/alDuncanson/Handler/discussions"


class ContactPanel(Container):
    """Contact panel for connecting to an agent endpoint."""

    ALLOW_MAXIMIZE = False

    BINDINGS = [
        Binding("h", "previous_tab", "← Tab", show=True, key_display="h/←"),
        Binding("l", "next_tab", "→ Tab", show=True, key_display="l/→"),
        Binding("left", "previous_tab", "Previous Tab", show=False),
        Binding("right", "next_tab", "Next Tab", show=False),
        Binding("enter", "focus_input", "Edit URL", show=True),
        Binding("b", "open_bug_report", "Bug", show=True),
        Binding("s", "open_sponsor", "Sponsor", show=True),
        Binding("d", "open_discuss", "Discuss", show=True),
    ]

    can_focus = True

    def check_action(self, action: str, parameters: tuple[object, ...]) -> bool | None:
        """Show/hide actions based on active tab context."""
        is_help = self._is_help_tab_active()
        if action in ("open_bug_report", "open_sponsor", "open_discuss"):
            return is_help
        if action == "focus_input":
            return not is_help
        return True

    def __init__(self, **kwargs: Any) -> None:
        super().__init__(**kwargs)
        self._version: str = "0.0.0"

    def compose(self) -> ComposeResult:
        with TabbedContent(id="contact-tabs"):
            with TabPane("Server", id="server-tab"):
                yield Vertical(
                    Horizontal(
                        Input(
                            placeholder="http://localhost:8000",
                            value="http://localhost:8000",
                            id="agent-url",
                        ),
                        Button("CONNECT", id="connect-btn"),
                        id="url-row",
                    ),
                    id="server-content",
                )
            with TabPane("Help", id="help-tab"):
                yield Vertical(
                    Static(id="version-info"),
                    Static("[dim]b[/dim] Report a bug:", classes="link-label"),
                    Link(REPORT_BUG_URL, url=REPORT_BUG_URL, id="report-bug-link"),
                    Static("[dim]s[/dim] Sponsor or donate:", classes="link-label"),
                    Link(SPONSOR_URL, url=SPONSOR_URL, id="sponsor-link"),
                    Static("[dim]d[/dim] Start a discussion:", classes="link-label"),
                    Link(DISCUSS_URL, url=DISCUSS_URL, id="discuss-link"),
                    id="help-content",
                )

    def on_mount(self) -> None:
        for widget in self.query("TabbedContent, Tabs, Tab, TabPane"):
            widget.can_focus = False
        self.query_one("#agent-url", Input).can_focus = False
        self.query_one("#connect-btn", Button).can_focus = False
        for link in self.query(Link):
            link.can_focus = False
        self._update_version_display()
        logger.debug("Contact panel mounted")

    @on(TabbedContent.TabActivated)
    def _on_tab_activated(self) -> None:
        """Refresh bindings when switching tabs."""
        self.refresh_bindings()

    def on_input_submitted(self, event: Input.Submitted) -> None:
        """Handle enter key in the URL input to connect."""
        connect_btn = self.query_one("#connect-btn", Button)
        self.post_message(Button.Pressed(connect_btn))

    def action_focus_input(self) -> None:
        """Focus the URL input field."""
        url_input = self.query_one("#agent-url", Input)
        url_input.can_focus = True
        url_input.focus()

    def on_descendant_blur(self) -> None:
        """Disable focus on input when it loses focus."""
        url_input = self.query_one("#agent-url", Input)
        url_input.can_focus = False

    def action_previous_tab(self) -> None:
        """Switch to the previous tab."""
        try:
            tabs_widget = self.query_one("#contact-tabs Tabs", Tabs)
            tabs_widget.action_previous_tab()
        except Exception:
            pass

    def action_next_tab(self) -> None:
        """Switch to the next tab."""
        try:
            tabs_widget = self.query_one("#contact-tabs Tabs", Tabs)
            tabs_widget.action_next_tab()
        except Exception:
            pass

    def _is_help_tab_active(self) -> bool:
        """Check if the Help tab is currently active."""
        try:
            tabs = self.query_one("#contact-tabs", TabbedContent)
            return tabs.active == "help-tab"
        except Exception:
            return False

    def action_open_bug_report(self) -> None:
        """Open the bug report URL."""
        if not self._is_help_tab_active():
            return
        webbrowser.open(REPORT_BUG_URL)

    def action_open_sponsor(self) -> None:
        """Open the sponsor URL."""
        if not self._is_help_tab_active():
            return
        webbrowser.open(SPONSOR_URL)

    def action_open_discuss(self) -> None:
        """Open the discuss URL."""
        if not self._is_help_tab_active():
            return
        webbrowser.open(DISCUSS_URL)

    def set_version(self, version: str) -> None:
        """Set the application version."""
        self._version = version
        self._update_version_display()

    def _update_version_display(self) -> None:
        """Update the version info display."""
        try:
            version_widget = self.query_one("#version-info", Static)
            version_widget.update(f"Handler v{self._version}")
        except Exception:
            pass

    def get_url(self) -> str:
        """Get the current agent URL from the input field."""
        url_input = self.query_one("#agent-url", Input)
        return url_input.value.strip()



"""Simple logging configuration for Handler.

Provides basic console logging with optional TUI log capture.
"""

import logging
import sys
from collections.abc import Callable
from dataclasses import dataclass
from datetime import datetime
from typing import Literal

LogLevel = Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]


@dataclass
class LogRecord:
    """A single log record for TUI display."""

    timestamp: datetime
    level: str
    name: str
    message: str


class TUILogHandler(logging.Handler):
    """Logging handler that captures records for TUI display.

    Stores formatted log lines and notifies a callback when new records arrive.
    """

    def __init__(
        self,
        max_records: int = 1000,
        callback: Callable[[str], None] | None = None,
    ) -> None:
        super().__init__()
        self.max_records = max_records
        self.lines: list[str] = []
        self.callback = callback

    def set_callback(self, callback: Callable[[str], None] | None) -> None:
        """Set or update the callback for new log lines."""
        self.callback = callback

    def emit(self, record: logging.LogRecord) -> None:
        """Handle a log record."""
        try:
            timestamp = datetime.fromtimestamp(record.created)
            time_str = timestamp.strftime("%H:%M:%S.%f")[:-3]
            short_name = record.name.split(".")[-1]
            message = self.format(record)

            line = f"{time_str} {record.levelname:>5} {short_name}: {message}"
            self.lines.append(line)

            if len(self.lines) > self.max_records:
                self.lines = self.lines[-self.max_records :]

            if self.callback:
                self.callback(line)

        except Exception:
            self.handleError(record)

    def get_lines(self) -> list[str]:
        """Get all stored log lines."""
        return list(self.lines)

    def clear(self) -> None:
        """Clear all stored lines."""
        self.lines.clear()


_tui_handler: TUILogHandler | None = None


def get_tui_log_handler() -> TUILogHandler:
    """Get or create the singleton TUI log handler."""
    global _tui_handler
    if _tui_handler is None:
        _tui_handler = TUILogHandler()
        _tui_handler.setFormatter(logging.Formatter("%(message)s"))
    return _tui_handler


def install_tui_log_handler(level: int = logging.DEBUG) -> TUILogHandler:
    """Install the TUI log handler on the root logger.

    Args:
        level: Minimum log level to capture

    Returns:
        The installed handler
    """
    handler = get_tui_log_handler()
    handler.setLevel(level)

    root_logger = logging.getLogger()

    if handler not in root_logger.handlers:
        root_logger.addHandler(handler)

    root_logger.setLevel(min(root_logger.level or level, level))

    return handler


def setup_logging(
    level: LogLevel = "INFO",
    show_time: bool = True,
) -> None:
    """Configure simple console logging for CLI.

    Args:
        level: Logging level
        show_time: Show timestamp in log output
    """
    format_str = "%(levelname)s: %(message)s"
    if show_time:
        format_str = "[%(asctime)s] " + format_str

    handler = logging.StreamHandler(sys.stderr)
    handler.setFormatter(logging.Formatter(format_str, datefmt="%H:%M:%S"))

    logging.basicConfig(
        level=level,
        handlers=[handler],
        force=True,
    )

    for lib in ["httpx", "httpcore", "uvicorn.access"]:
        logging.getLogger(lib).setLevel(logging.WARNING)


def get_logger(name: str) -> logging.Logger:
    """Get a logger for a module.

    Args:
        name: Logger name (typically __name__)

    Returns:
        Configured logger instance
    """
    return logging.getLogger(name)



"""A2A protocol service layer.

Provides a unified interface for A2A operations, shared between the CLI and TUI.
"""

import uuid
from dataclasses import dataclass
from typing import AsyncIterator

import httpx
from a2a.client import A2ACardResolver, Client, ClientConfig, ClientFactory
from a2a.types import (
    AgentCard,
    GetTaskPushNotificationConfigParams,
    Message,
    Part,
    PushNotificationConfig,
    Role,
    Task,
    TaskArtifactUpdateEvent,
    TaskIdParams,
    TaskPushNotificationConfig,
    TaskQueryParams,
    TaskState,
    TaskStatusUpdateEvent,
    TextPart,
    TransportProtocol,
)

from a2a_handler.auth import AuthCredentials
from a2a_handler.common import get_logger

logger = get_logger(__name__)

TERMINAL_TASK_STATES = {
    TaskState.completed,
    TaskState.canceled,
    TaskState.failed,
    TaskState.rejected,
}


@dataclass
class SendResult:
    """Result from sending a message to an agent.

    This is a Handler convenience wrapper around SDK types (Task, Message).
    All A2A protocol data is accessible via the `task` and `message` fields.
    """

    task: Task | None = None
    message: Message | None = None
    text: str = ""

    @property
    def context_id(self) -> str | None:
        """Get context_id from the underlying SDK type."""
        if self.task:
            return self.task.context_id
        if self.message:
            return self.message.context_id
        return None

    @property
    def task_id(self) -> str | None:
        """Get task_id from the underlying SDK type."""
        if self.task:
            return self.task.id
        if self.message:
            return self.message.task_id
        return None

    @property
    def state(self) -> TaskState | None:
        """Get task state from the underlying SDK type."""
        if self.task and self.task.status:
            return self.task.status.state
        return None

    @property
    def is_complete(self) -> bool:
        """Check if the task reached a terminal state."""
        return self.state in TERMINAL_TASK_STATES if self.state else False

    @property
    def needs_input(self) -> bool:
        """Check if the task is waiting for user input."""
        return self.state == TaskState.input_required if self.state else False

    @property
    def needs_auth(self) -> bool:
        """Check if the task requires authentication."""
        return self.state == TaskState.auth_required if self.state else False


@dataclass
class StreamEvent:
    """A single event from a streaming response.

    This is a Handler convenience wrapper around SDK streaming event types.
    The original SDK event is accessible via `status` or `artifact` fields.
    """

    event_type: str
    task: Task | None = None
    message: Message | None = None
    status: TaskStatusUpdateEvent | None = None
    artifact: TaskArtifactUpdateEvent | None = None
    text: str = ""

    @property
    def context_id(self) -> str | None:
        """Get context_id from the underlying SDK type."""
        if self.task:
            return self.task.context_id
        if self.message:
            return self.message.context_id
        if self.status:
            return self.status.context_id
        if self.artifact:
            return self.artifact.context_id
        return None

    @property
    def task_id(self) -> str | None:
        """Get task_id from the underlying SDK type."""
        if self.task:
            return self.task.id
        if self.message:
            return self.message.task_id
        if self.status:
            return self.status.task_id
        if self.artifact:
            return self.artifact.task_id
        return None

    @property
    def state(self) -> TaskState | None:
        """Get task state from the underlying SDK type."""
        if self.task and self.task.status:
            return self.task.status.state
        if self.status and self.status.status:
            return self.status.status.state
        return None


@dataclass
class TaskResult:
    """Result from a task operation (get/cancel).

    This is a Handler convenience wrapper around the SDK Task type.
    All A2A protocol data is accessible via the `task` field.
    """

    task: Task
    text: str = ""

    @property
    def task_id(self) -> str:
        """Get task_id from the underlying SDK type."""
        return self.task.id

    @property
    def context_id(self) -> str:
        """Get context_id from the underlying SDK type."""
        return self.task.context_id

    @property
    def state(self) -> TaskState:
        """Get task state from the underlying SDK type."""
        return self.task.status.state if self.task.status else TaskState.unknown


def extract_text_from_message_parts(message_parts: list[Part] | None) -> str:
    """Extract text content from message parts."""
    if not message_parts:
        return ""

    extracted_texts = []
    for part in message_parts:
        if isinstance(part.root, TextPart):
            extracted_texts.append(part.root.text)

    return "\n".join(text for text in extracted_texts if text)


def extract_text_from_task(task: Task) -> str:
    """Extract text from task artifacts, falling back to history if no artifacts."""
    extracted_texts = []

    if task.artifacts:
        for artifact in task.artifacts:
            if artifact.parts:
                extracted_texts.append(extract_text_from_message_parts(artifact.parts))

    # Only check history if no artifacts found (avoids duplication)
    if not extracted_texts and task.history:
        for message in task.history:
            if message.role == Role.agent and message.parts:
                extracted_texts.append(extract_text_from_message_parts(message.parts))

    return "\n".join(text for text in extracted_texts if text)


class A2AService:
    """High-level service for A2A protocol operations.

    Wraps the a2a-sdk Client and provides a simplified interface
    for common operations. Designed to be shared between CLI and TUI.
    """

    def __init__(
        self,
        http_client: httpx.AsyncClient,
        agent_url: str,
        enable_streaming: bool = True,
        push_notification_url: str | None = None,
        push_notification_token: str | None = None,
        credentials: AuthCredentials | None = None,
    ) -> None:
        """Initialize the A2A service.

        Args:
            http_client: Async HTTP client to use for requests
            agent_url: Base URL of the A2A agent
            enable_streaming: Whether to prefer streaming when available
            push_notification_url: Optional webhook URL for push notifications
            push_notification_token: Optional token for push notification auth
            credentials: Optional authentication credentials
        """
        self.http_client = http_client
        self.agent_url = agent_url
        self.enable_streaming = enable_streaming
        self.push_notification_url = push_notification_url
        self.push_notification_token = push_notification_token
        self.credentials = credentials
        self._cached_client: Client | None = None
        self._cached_agent_card: AgentCard | None = None
        self._applied_auth_headers: set[str] = set()

        if credentials:
            self.set_credentials(credentials)

    def set_credentials(self, credentials: AuthCredentials) -> None:
        """Set or update authentication credentials.

        Args:
            credentials: Authentication credentials to apply
        """
        for header_name in self._applied_auth_headers:
            self.http_client.headers.pop(header_name, None)
        self._applied_auth_headers.clear()

        self.credentials = credentials
        auth_headers = credentials.to_headers()
        self.http_client.headers.update(auth_headers)
        self._applied_auth_headers = set(auth_headers.keys())
        logger.debug("Applied authentication headers: %s", list(auth_headers.keys()))

    async def get_card(self) -> AgentCard:
        """Fetch and cache the agent card.

        Returns:
            The agent's card with metadata and capabilities
        """
        if self._cached_agent_card is None:
            logger.info("Fetching agent card from %s", self.agent_url)
            card_resolver = A2ACardResolver(self.http_client, self.agent_url)
            self._cached_agent_card = await card_resolver.get_agent_card()
            logger.info("Connected to agent: %s", self._cached_agent_card.name)
        return self._cached_agent_card

    async def _get_or_create_client(self) -> Client:
        """Get or create the A2A client.

        Returns:
            Configured A2A client instance
        """
        if self._cached_client is None:
            agent_card = await self.get_card()

            push_notification_configs: list[PushNotificationConfig] = []
            if self.push_notification_url:
                push_notification_configs.append(
                    PushNotificationConfig(
                        url=self.push_notification_url,
                        token=self.push_notification_token,
                    )
                )
                logger.info(
                    "Push notification configured: %s", self.push_notification_url
                )

            client_config = ClientConfig(
                httpx_client=self.http_client,
                supported_transports=[TransportProtocol.jsonrpc],
                streaming=self.enable_streaming,
                push_notification_configs=push_notification_configs,
            )

            client_factory = ClientFactory(client_config)
            self._cached_client = client_factory.create(agent_card)
            logger.debug("Created A2A client for %s", agent_card.name)

        return self._cached_client

    @property
    def supports_streaming(self) -> bool:
        """Check if the agent supports streaming."""
        if self._cached_agent_card and self._cached_agent_card.capabilities:
            return bool(self._cached_agent_card.capabilities.streaming)
        return False

    @property
    def supports_push_notifications(self) -> bool:
        """Check if the agent supports push notifications."""
        if self._cached_agent_card and self._cached_agent_card.capabilities:
            return bool(self._cached_agent_card.capabilities.push_notifications)
        return False

    def _build_user_message(
        self,
        message_text: str,
        context_id: str | None = None,
        task_id: str | None = None,
    ) -> Message:
        """Build a user message.

        Args:
            message_text: Message content
            context_id: Optional context ID for conversation continuity
            task_id: Optional task ID to continue

        Returns:
            Properly formatted Message object
        """
        return Message(
            message_id=str(uuid.uuid4()),
            role=Role.user,
            parts=[Part(root=TextPart(text=message_text))],
            context_id=context_id,
            task_id=task_id,
        )

    async def send(
        self,
        message_text: str,
        context_id: str | None = None,
        task_id: str | None = None,
    ) -> SendResult:
        """Send a message to the agent and wait for completion.

        This method collects all streaming events and returns the final result.

        Args:
            message_text: Message to send
            context_id: Optional context ID for conversation continuity
            task_id: Optional task ID to continue

        Returns:
            SendResult with task state, extracted text, and IDs
        """
        client = await self._get_or_create_client()
        user_message = self._build_user_message(message_text, context_id, task_id)

        truncated_message = (
            message_text[:50] if len(message_text) > 50 else message_text
        )
        logger.info("Sending message: %s", truncated_message)

        result = SendResult()

        async for event in client.send_message(user_message):
            if isinstance(event, Message):
                result.message = event
                result.text = extract_text_from_message_parts(event.parts)
                logger.debug("Received message response")
            elif isinstance(event, tuple):
                received_task, task_update = event
                result.task = received_task
                logger.debug(
                    "Received task update: %s",
                    received_task.status.state if received_task.status else "unknown",
                )

        if result.task:
            result.text = extract_text_from_task(result.task)

        logger.info("Send complete: task_id=%s, state=%s", result.task_id, result.state)
        return result

    async def stream(
        self,
        message_text: str,
        context_id: str | None = None,
        task_id: str | None = None,
    ) -> AsyncIterator[StreamEvent]:
        """Send a message and stream responses as they arrive.

        Args:
            message_text: Message to send
            context_id: Optional context ID for conversation continuity
            task_id: Optional task ID to continue

        Yields:
            StreamEvent objects as they are received
        """
        client = await self._get_or_create_client()
        user_message = self._build_user_message(message_text, context_id, task_id)

        truncated_message = (
            message_text[:50] if len(message_text) > 50 else message_text
        )
        logger.info("Streaming message: %s", truncated_message)

        async for event in client.send_message(user_message):
            if isinstance(event, Message):
                yield StreamEvent(
                    event_type="message",
                    message=event,
                    text=extract_text_from_message_parts(event.parts),
                )
            elif isinstance(event, tuple):
                received_task, task_update = event
                if isinstance(task_update, TaskStatusUpdateEvent):
                    status_message_text = ""
                    if task_update.status and task_update.status.message:
                        status_message_text = str(task_update.status.message)
                    yield StreamEvent(
                        event_type="status",
                        task=received_task,
                        status=task_update,
                        text=status_message_text,
                    )
                elif isinstance(task_update, TaskArtifactUpdateEvent):
                    artifact_text = ""
                    if task_update.artifact and task_update.artifact.parts:
                        artifact_text = extract_text_from_message_parts(
                            task_update.artifact.parts
                        )
                    yield StreamEvent(
                        event_type="artifact",
                        task=received_task,
                        artifact=task_update,
                        text=artifact_text,
                    )
                else:
                    yield StreamEvent(
                        event_type="task",
                        task=received_task,
                        text=extract_text_from_task(received_task),
                    )

    async def get_task(
        self,
        task_id: str,
        history_length: int | None = None,
    ) -> TaskResult:
        """Get the current state of a task.

        Args:
            task_id: ID of the task to retrieve
            history_length: Optional number of history messages to include

        Returns:
            TaskResult with task state and details
        """
        client = await self._get_or_create_client()

        query_params = TaskQueryParams(id=task_id, history_length=history_length)
        logger.info("Getting task: %s", task_id)

        task = await client.get_task(query_params)

        return TaskResult(
            task=task,
            text=extract_text_from_task(task),
        )

    async def cancel_task(self, task_id: str) -> TaskResult:
        """Cancel a running task.

        Args:
            task_id: ID of the task to cancel

        Returns:
            TaskResult with updated task state
        """
        client = await self._get_or_create_client()

        task_id_params = TaskIdParams(id=task_id)
        logger.info("Canceling task: %s", task_id)

        task = await client.cancel_task(task_id_params)

        return TaskResult(
            task=task,
            text=extract_text_from_task(task),
        )

    async def resubscribe(self, task_id: str) -> AsyncIterator[StreamEvent]:
        """Resubscribe to a task's event stream.

        Args:
            task_id: ID of the task to resubscribe to

        Yields:
            StreamEvent objects as they are received
        """
        client = await self._get_or_create_client()

        task_id_params = TaskIdParams(id=task_id)
        logger.info("Resubscribing to task: %s", task_id)

        async for event in client.resubscribe(task_id_params):
            received_task, task_update = event
            if isinstance(task_update, TaskStatusUpdateEvent):
                yield StreamEvent(
                    event_type="status",
                    task=received_task,
                    status=task_update,
                )
            elif isinstance(task_update, TaskArtifactUpdateEvent):
                artifact_text = ""
                if task_update.artifact and task_update.artifact.parts:
                    artifact_text = extract_text_from_message_parts(
                        task_update.artifact.parts
                    )
                yield StreamEvent(
                    event_type="artifact",
                    task=received_task,
                    artifact=task_update,
                    text=artifact_text,
                )
            else:
                yield StreamEvent(
                    event_type="task",
                    task=received_task,
                    text=extract_text_from_task(received_task),
                )

    async def set_push_config(
        self,
        task_id: str,
        webhook_url: str,
        authentication_token: str | None = None,
    ) -> TaskPushNotificationConfig:
        """Set push notification configuration for a task.

        Args:
            task_id: ID of the task
            webhook_url: Webhook URL to receive notifications
            authentication_token: Optional authentication token

        Returns:
            The created push notification configuration
        """
        client = await self._get_or_create_client()

        push_config = TaskPushNotificationConfig(
            task_id=task_id,
            push_notification_config=PushNotificationConfig(
                url=webhook_url,
                token=authentication_token,
            ),
        )
        logger.info("Setting push config for task %s: %s", task_id, webhook_url)

        return await client.set_task_callback(push_config)

    async def get_push_config(
        self,
        task_id: str,
        config_id: str | None = None,
    ) -> TaskPushNotificationConfig:
        """Get push notification configuration for a task.

        Args:
            task_id: ID of the task
            config_id: Optional specific config ID to retrieve

        Returns:
            The push notification configuration
        """
        client = await self._get_or_create_client()

        params = GetTaskPushNotificationConfigParams(
            id=task_id,
            push_notification_config_id=config_id,
        )
        logger.info("Getting push config for task %s", task_id)

        return await client.get_task_callback(params)



"""Task commands for managing A2A tasks."""

import asyncio
from typing import Optional

import rich_click as click

from a2a_handler.auth import AuthCredentials, create_api_key_auth, create_bearer_auth
from a2a_handler.common import Output, get_logger
from a2a_handler.service import A2AService, TaskResult
from a2a_handler.session import get_credentials

from ._helpers import build_http_client, handle_client_error

log = get_logger(__name__)


@click.group()
def task() -> None:
    """Manage A2A tasks."""
    pass


@task.command("get")
@click.argument("agent_url")
@click.argument("task_id")
@click.option(
    "--history-length", "-n", type=int, help="Number of history messages to include"
)
@click.option("--bearer", "-b", "bearer_token", help="Bearer token (overrides saved)")
@click.option("--api-key", "-k", help="API key (overrides saved)")
def task_get(
    agent_url: str,
    task_id: str,
    history_length: Optional[int],
    bearer_token: Optional[str],
    api_key: Optional[str],
) -> None:
    """Retrieve the current status of a task."""
    log.info("Getting task %s from %s", task_id, agent_url)

    credentials: AuthCredentials | None = None
    if bearer_token:
        credentials = create_bearer_auth(bearer_token)
    elif api_key:
        credentials = create_api_key_auth(api_key)
    else:
        credentials = get_credentials(agent_url)

    async def do_get() -> None:
        output = Output()
        try:
            async with build_http_client() as http_client:
                service = A2AService(http_client, agent_url, credentials=credentials)
                result = await service.get_task(task_id, history_length)
                _format_task_result(result, output)
        except Exception as e:
            handle_client_error(e, agent_url, output)
            raise click.Abort()

    asyncio.run(do_get())


@task.command("cancel")
@click.argument("agent_url")
@click.argument("task_id")
@click.option("--bearer", "-b", "bearer_token", help="Bearer token (overrides saved)")
@click.option("--api-key", "-k", help="API key (overrides saved)")
def task_cancel(
    agent_url: str,
    task_id: str,
    bearer_token: Optional[str],
    api_key: Optional[str],
) -> None:
    """Request cancellation of a task."""
    log.info("Canceling task %s at %s", task_id, agent_url)

    credentials: AuthCredentials | None = None
    if bearer_token:
        credentials = create_bearer_auth(bearer_token)
    elif api_key:
        credentials = create_api_key_auth(api_key)
    else:
        credentials = get_credentials(agent_url)

    async def do_cancel() -> None:
        output = Output()
        try:
            async with build_http_client() as http_client:
                service = A2AService(http_client, agent_url, credentials=credentials)

                output.dim(f"Canceling task {task_id}...")

                result = await service.cancel_task(task_id)
                _format_task_result(result, output)

                output.success("Task canceled")

        except Exception as e:
            handle_client_error(e, agent_url, output)
            raise click.Abort()

    asyncio.run(do_cancel())


@task.command("resubscribe")
@click.argument("agent_url")
@click.argument("task_id")
@click.option("--bearer", "-b", "bearer_token", help="Bearer token (overrides saved)")
@click.option("--api-key", "-k", help="API key (overrides saved)")
def task_resubscribe(
    agent_url: str,
    task_id: str,
    bearer_token: Optional[str],
    api_key: Optional[str],
) -> None:
    """Resubscribe to a task's SSE stream after disconnection."""
    log.info("Resubscribing to task %s at %s", task_id, agent_url)

    credentials: AuthCredentials | None = None
    if bearer_token:
        credentials = create_bearer_auth(bearer_token)
    elif api_key:
        credentials = create_api_key_auth(api_key)
    else:
        credentials = get_credentials(agent_url)

    async def do_resubscribe() -> None:
        output = Output()
        try:
            async with build_http_client() as http_client:
                service = A2AService(http_client, agent_url, credentials=credentials)

                output.dim(f"Resubscribing to task {task_id}...")

                async for event in service.resubscribe(task_id):
                    if event.event_type == "status":
                        output.state(
                            "Status",
                            event.state.value if event.state else "unknown",
                        )
                    elif event.text:
                        output.line(event.text)

        except Exception as e:
            handle_client_error(e, agent_url, output)
            raise click.Abort()

    asyncio.run(do_resubscribe())


def _format_task_result(result: TaskResult, output: Output) -> None:
    """Format and display a task result."""
    output.blank()
    output.field("Task ID", result.task_id, dim_value=True)
    output.state("State", result.state.value)
    if result.context_id:
        output.field("Context ID", result.context_id, dim_value=True)

    if result.text:
        output.blank()
        output.markdown(result.text)


@task.group("notification")
def task_notification() -> None:
    """Manage push notification configurations for tasks."""
    pass


@task_notification.command("set")
@click.argument("agent_url")
@click.argument("task_id")
@click.option("--url", "-u", required=True, help="Webhook URL to receive notifications")
@click.option("--token", "-t", help="Authentication token for the webhook")
@click.option("--bearer", "-b", "bearer_token", help="Bearer token (overrides saved)")
@click.option("--api-key", "-k", help="API key (overrides saved)")
def notification_set(
    agent_url: str,
    task_id: str,
    url: str,
    token: Optional[str],
    bearer_token: Optional[str],
    api_key: Optional[str],
) -> None:
    """Configure a push notification webhook for a task."""
    log.info("Setting push config for task %s at %s", task_id, agent_url)

    credentials: AuthCredentials | None = None
    if bearer_token:
        credentials = create_bearer_auth(bearer_token)
    elif api_key:
        credentials = create_api_key_auth(api_key)
    else:
        credentials = get_credentials(agent_url)

    async def do_set() -> None:
        output = Output()
        try:
            async with build_http_client() as http_client:
                service = A2AService(http_client, agent_url, credentials=credentials)

                output.dim(f"Setting notification config for task {task_id}...")

                config = await service.set_push_config(task_id, url, token)

                output.success("Push notification config set")
                output.field("Task ID", config.task_id)
                if config.push_notification_config:
                    pnc = config.push_notification_config
                    output.field("URL", pnc.url)
                    if pnc.token:
                        output.field("Token", f"{pnc.token[:20]}...")
                    if pnc.id:
                        output.field("Config ID", pnc.id)

        except Exception as e:
            handle_client_error(e, agent_url, output)
            raise click.Abort()

    asyncio.run(do_set())


@task_notification.command("get")
@click.argument("agent_url")
@click.argument("task_id")
@click.option("--config-id", "-c", help="Specific push notification config ID")
@click.option("--bearer", "-b", "bearer_token", help="Bearer token (overrides saved)")
@click.option("--api-key", "-k", help="API key (overrides saved)")
def notification_get(
    agent_url: str,
    task_id: str,
    config_id: Optional[str],
    bearer_token: Optional[str],
    api_key: Optional[str],
) -> None:
    """Get the push notification configuration for a task."""
    log.info("Getting push config for task %s at %s", task_id, agent_url)

    credentials: AuthCredentials | None = None
    if bearer_token:
        credentials = create_bearer_auth(bearer_token)
    elif api_key:
        credentials = create_api_key_auth(api_key)
    else:
        credentials = get_credentials(agent_url)

    async def do_get() -> None:
        output = Output()
        try:
            async with build_http_client() as http_client:
                service = A2AService(http_client, agent_url, credentials=credentials)

                config = await service.get_push_config(task_id, config_id)

                output.field("Task ID", config.task_id)
                if config.push_notification_config:
                    pnc = config.push_notification_config
                    output.field("URL", pnc.url)
                    if pnc.token:
                        output.field("Token", f"{pnc.token[:20]}...")
                    if pnc.id:
                        output.field("Config ID", pnc.id)

        except Exception as e:
            handle_client_error(e, agent_url, output)
            raise click.Abort()

    asyncio.run(do_get())



"""Auth commands for managing authentication credentials."""

from typing import Optional

import rich_click as click

from a2a_handler.auth import AuthType, create_api_key_auth, create_bearer_auth
from a2a_handler.common import Output
from a2a_handler.session import clear_credentials, get_credentials, set_credentials


@click.group()
def auth() -> None:
    """Manage authentication credentials for agents."""
    pass


@auth.command("set")
@click.argument("agent_url")
@click.option("--bearer", "-b", "bearer_token", help="Bearer token for authentication")
@click.option("--api-key", "-k", "api_key", help="API key for authentication")
@click.option(
    "--api-key-header",
    default="X-API-Key",
    help="Header name for API key (default: X-API-Key)",
)
def auth_set(
    agent_url: str,
    bearer_token: Optional[str],
    api_key: Optional[str],
    api_key_header: str,
) -> None:
    """Set authentication credentials for an agent.

    Provide either --bearer or --api-key (not both).
    """
    output = Output()
    if bearer_token and api_key:
        output.error("Provide either --bearer or --api-key, not both")
        raise click.Abort()

    if not bearer_token and not api_key:
        output.error("Provide --bearer or --api-key")
        raise click.Abort()

    if bearer_token:
        credentials = create_bearer_auth(bearer_token)
        auth_type_display = "Bearer token"
    else:
        credentials = create_api_key_auth(api_key or "", header_name=api_key_header)
        auth_type_display = f"API key (header: {api_key_header})"

    set_credentials(agent_url, credentials)

    output.success(f"Set {auth_type_display} for {agent_url}")


@auth.command("show")
@click.argument("agent_url")
def auth_show(agent_url: str) -> None:
    """Show authentication credentials for an agent."""
    output = Output()
    credentials = get_credentials(agent_url)

    output.header(f"Auth for {agent_url}")

    if not credentials:
        output.dim("No credentials configured")
        return

    output.field("Type", credentials.auth_type.value)
    masked_value = (
        f"{credentials.value[:4]}...{credentials.value[-4:]}"
        if len(credentials.value) > 8
        else "****"
    )
    output.field("Value", masked_value)

    if credentials.auth_type == AuthType.API_KEY:
        output.field("Header", credentials.header_name or "X-API-Key")


@auth.command("clear")
@click.argument("agent_url")
def auth_clear(agent_url: str) -> None:
    """Clear authentication credentials for an agent."""
    output = Output()
    clear_credentials(agent_url)
    output.success(f"Cleared credentials for {agent_url}")



"""Agent card building and configuration."""

from a2a.types import (
    AgentCapabilities,
    AgentCard,
    AgentSkill,
    APIKeySecurityScheme,
    In,
    SecurityScheme,
)
from google.adk.agents.llm_agent import Agent

from a2a_handler.common import get_logger

logger = get_logger(__name__)


def build_agent_card(
    agent: Agent,
    host: str,
    port: int,
    require_auth: bool = False,
) -> AgentCard:
    """Build an AgentCard with streaming and push notification capabilities.

    Args:
        agent: The ADK agent
        host: Host address for the RPC URL
        port: Port number for the RPC URL
        require_auth: Whether to require API key authentication

    Returns:
        Configured AgentCard with capabilities enabled
    """
    agent_capabilities = AgentCapabilities(
        streaming=True,
        push_notifications=True,
    )

    skills = [
        AgentSkill(
            id="handler_assistant",
            name="Handler Assistant",
            description="Helps with Handler CLI commands, TUI usage, and troubleshooting",
            tags=["handler", "cli", "tui", "help"],
            examples=[
                "How do I send a message with Handler?",
                "What CLI commands are available?",
                "How do I validate an agent card?",
            ],
        ),
    ]

    display_host = "localhost" if host == "0.0.0.0" else host
    rpc_endpoint_url = f"http://{display_host}:{port}/"

    logger.debug("Building agent card with RPC URL: %s", rpc_endpoint_url)

    security_schemes: dict[str, SecurityScheme] | None = None
    security: list[dict[str, list[str]]] | None = None

    if require_auth:
        api_key_scheme = SecurityScheme(
            root=APIKeySecurityScheme(
                type="apiKey",
                name="X-API-Key",
                in_=In.header,
            )
        )
        security_schemes = {"apiKey": api_key_scheme}
        security = [{"apiKey": []}]
        logger.info("API key authentication enabled")

    return AgentCard(
        name=agent.name,
        description=agent.description or "Handler A2A agent",
        url=rpc_endpoint_url,
        version="1.0.0",
        capabilities=agent_capabilities,
        skills=skills,
        default_input_modes=["text/plain"],
        default_output_modes=["text/plain"],
        security_schemes=security_schemes,
        security=security,
    )



"""Artifacts panel component for viewing artifact history and details."""

from __future__ import annotations

import json
from datetime import datetime
from typing import TYPE_CHECKING

from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Container, Horizontal, VerticalScroll
from textual.message import Message as TextualMessage
from textual.reactive import reactive
from textual.widgets import Collapsible, Label, ListItem, ListView, Static

from a2a_handler.common import get_logger

if TYPE_CHECKING:
    from a2a.types import Artifact

logger = get_logger(__name__)


class ArtifactEntry:
    """Represents an artifact entry for display."""

    def __init__(
        self,
        artifact: Artifact,
        task_id: str,
        context_id: str,
        received_at: datetime | None = None,
    ) -> None:
        self.artifact = artifact
        self.task_id = task_id
        self.context_id = context_id
        self.received_at = received_at or datetime.now()

    @property
    def artifact_id(self) -> str:
        return self.artifact.artifact_id or "unnamed"

    @property
    def name(self) -> str | None:
        return self.artifact.name

    @property
    def description(self) -> str | None:
        return self.artifact.description


class ArtifactListItem(ListItem):
    """A single artifact item in the list."""

    def __init__(
        self,
        entry: ArtifactEntry,
        *,
        name: str | None = None,
        id: str | None = None,
        classes: str | None = None,
        disabled: bool = False,
    ) -> None:
        super().__init__(name=name, id=id, classes=classes, disabled=disabled)
        self.entry = entry

    def compose(self) -> ComposeResult:
        time_str = self.entry.received_at.strftime("%H:%M:%S")
        artifact_id_short = self.entry.artifact_id[:8]
        display_name = self.entry.name or artifact_id_short
        yield Label(f"{time_str}  {display_name}")


class ArtifactDetailPanel(VerticalScroll):
    """Panel showing detailed information about the selected artifact."""

    def compose(self) -> ComposeResult:
        yield Static(
            "Select an artifact to view details", id="artifact-detail-placeholder"
        )
        yield Container(id="artifact-detail-content", classes="hidden")

    def _field(
        self, label: str, value: str, value_class: str = "artifact-value"
    ) -> Horizontal:
        """Create a label-value field row."""
        return Horizontal(
            Label(f"{label}: ", classes="artifact-label"),
            Label(value, classes=value_class),
            classes="artifact-field-row",
        )

    def _section_heading(self, title: str) -> Static:
        """Create a section heading."""
        return Static(title, classes="section-heading")

    def show_artifact(self, entry: ArtifactEntry | None) -> None:
        placeholder = self.query_one("#artifact-detail-placeholder", Static)
        content = self.query_one("#artifact-detail-content", Container)

        if entry is None:
            placeholder.remove_class("hidden")
            content.add_class("hidden")
            return

        placeholder.add_class("hidden")
        content.remove_class("hidden")
        content.remove_children()

        artifact = entry.artifact

        content.mount(self._section_heading("Metadata"))
        content.mount(
            self._field("Artifact ID", entry.artifact_id),
            self._field("Task ID", entry.task_id),
            self._field("Context ID", entry.context_id),
            self._field("Received", entry.received_at.strftime("%Y-%m-%d %H:%M:%S")),
        )

        if entry.name:
            content.mount(self._field("Name", entry.name))

        if entry.description:
            content.mount(self._field("Description", entry.description))

        if artifact.parts:
            content.mount(self._section_heading("Content"))
            from a2a_handler.service import extract_text_from_message_parts

            text = extract_text_from_message_parts(artifact.parts)
            if text:
                content.mount(Static(text, classes="artifact-content"))

        content.mount(self._section_heading("Raw JSON"))
        raw_json = json.dumps(artifact.model_dump(), indent=2, default=str)
        content.mount(
            Collapsible(
                Static(raw_json),
                title="View JSON",
                collapsed=True,
            )
        )


class ArtifactsPanel(Container):
    """Panel with split view: artifact list on left, details on right."""

    BINDINGS = [
        Binding("j", "cursor_down", "↓ Select", show=True, key_display="j/↓"),
        Binding("k", "cursor_up", "↑ Select", show=True, key_display="k/↑"),
        Binding("down", "cursor_down", "Down", show=False),
        Binding("up", "cursor_up", "Up", show=False),
        Binding("ctrl+d", "scroll_detail_down", "½ Page ↓", show=True),
        Binding("ctrl+u", "scroll_detail_up", "½ Page ↑", show=True),
        Binding("enter", "select_artifact", "View", show=True),
        Binding("y", "copy_artifact_id", "Copy ID", show=True),
        Binding("Y", "copy_task_id", "Copy Task", show=True),
    ]

    can_focus = False

    selected_index: reactive[int] = reactive(0)
    _artifacts: list[ArtifactEntry] = []

    class ArtifactSelected(TextualMessage):
        """Posted when an artifact is selected."""

        def __init__(self, entry: ArtifactEntry) -> None:
            super().__init__()
            self.entry = entry

    def compose(self) -> ComposeResult:
        with Horizontal(id="artifacts-split"):
            yield ListView(id="artifacts-list")
            yield ArtifactDetailPanel(id="artifact-detail")

    def on_mount(self) -> None:
        self._artifacts = []
        for widget in self.query("ListView, ArtifactDetailPanel"):
            widget.can_focus = False
        logger.debug("Artifacts panel mounted")

    def _get_list_view(self) -> ListView:
        return self.query_one("#artifacts-list", ListView)

    def _get_detail_panel(self) -> ArtifactDetailPanel:
        return self.query_one("#artifact-detail", ArtifactDetailPanel)

    def add_artifact(self, artifact: Artifact, task_id: str, context_id: str) -> None:
        """Add an artifact to the list."""
        entry = ArtifactEntry(artifact, task_id, context_id)
        self._artifacts.insert(0, entry)
        list_view = self._get_list_view()
        list_view.insert(0, [ArtifactListItem(entry)])
        logger.debug("Added artifact %s to artifacts panel", entry.artifact_id[:8])

        if len(self._artifacts) == 1:
            self._update_detail()

    def update_artifact(
        self, artifact: Artifact, task_id: str, context_id: str
    ) -> None:
        """Update an existing artifact or add if new."""
        artifact_id = artifact.artifact_id or "unnamed"
        for i, entry in enumerate(self._artifacts):
            if entry.artifact_id == artifact_id and entry.task_id == task_id:
                self._artifacts[i] = ArtifactEntry(
                    artifact, task_id, context_id, entry.received_at
                )
                list_view = self._get_list_view()
                children = list(list_view.children)
                if i < len(children):
                    old_item = children[i]
                    new_item = ArtifactListItem(self._artifacts[i])
                    old_item.remove()
                    list_view.insert(i, [new_item])

                if i == self.selected_index:
                    self._update_detail()
                logger.debug("Updated artifact %s", artifact_id[:8])
                return

        self.add_artifact(artifact, task_id, context_id)

    def _update_detail(self) -> None:
        """Update the detail panel with the currently selected artifact."""
        detail = self._get_detail_panel()
        if 0 <= self.selected_index < len(self._artifacts):
            detail.show_artifact(self._artifacts[self.selected_index])
        else:
            detail.show_artifact(None)

    def on_list_view_selected(self, event: ListView.Selected) -> None:
        """Handle list item selection."""
        list_view = self._get_list_view()
        if event.item and event.item in list_view.children:
            self.selected_index = list_view.children.index(event.item)
            self._update_detail()

    def on_list_view_highlighted(self, event: ListView.Highlighted) -> None:
        """Handle list item highlight (cursor movement)."""
        list_view = self._get_list_view()
        if event.item and event.item in list_view.children:
            self.selected_index = list_view.children.index(event.item)
            self._update_detail()

    def action_cursor_down(self) -> None:
        list_view = self._get_list_view()
        list_view.action_cursor_down()

    def action_cursor_up(self) -> None:
        list_view = self._get_list_view()
        list_view.action_cursor_up()

    def action_scroll_detail_down(self) -> None:
        """Scroll the detail panel down by half a page."""
        detail = self._get_detail_panel()
        detail.scroll_relative(y=detail.size.height // 2, animate=False)

    def action_scroll_detail_up(self) -> None:
        """Scroll the detail panel up by half a page."""
        detail = self._get_detail_panel()
        detail.scroll_relative(y=-(detail.size.height // 2), animate=False)

    def action_select_artifact(self) -> None:
        list_view = self._get_list_view()
        list_view.action_select_cursor()

    def action_copy_artifact_id(self) -> None:
        """Copy the selected artifact ID to clipboard."""
        if 0 <= self.selected_index < len(self._artifacts):
            artifact_id = self._artifacts[self.selected_index].artifact_id
            self.app.copy_to_clipboard(artifact_id)
            self.notify(f"Copied artifact ID: {artifact_id[:16]}...")

    def action_copy_task_id(self) -> None:
        """Copy the selected task ID to clipboard."""
        if 0 <= self.selected_index < len(self._artifacts):
            task_id = self._artifacts[self.selected_index].task_id
            self.app.copy_to_clipboard(task_id)
            self.notify(f"Copied task ID: {task_id[:16]}...")

    def get_selected_artifact(self) -> ArtifactEntry | None:
        """Get the currently selected artifact entry."""
        if 0 <= self.selected_index < len(self._artifacts):
            return self._artifacts[self.selected_index]
        return None

    def clear(self) -> None:
        """Clear all artifacts."""
        self._artifacts = []
        list_view = self._get_list_view()
        list_view.clear()
        self._update_detail()
        logger.info("Cleared artifacts panel")



"""A2A server agent with streaming and push notifications.

Provides a local A2A-compatible agent server for testing and development.
"""

import asyncio
import os

import uvicorn

from a2a_handler.common import get_logger, setup_logging

from .agent import DEFAULT_OLLAMA_MODEL, create_llm_agent
from .app import create_a2a_application, generate_api_key
from .card import build_agent_card
from .ollama import check_ollama_model, prompt_ollama_pull

setup_logging(level="INFO")
logger = get_logger(__name__)


def run_server(
    host: str,
    port: int,
    require_auth: bool = False,
    api_key: str | None = None,
    model: str | None = None,
) -> None:
    """Start the A2A server agent.

    Args:
        host: Host address to bind to
        port: Port number to bind to
        require_auth: Whether to require API key authentication
        api_key: Specific API key to use (generated if not provided and auth required)
        model: Ollama model identifier (e.g., 'llama3.2:1b')
    """
    effective_model = model or os.getenv("OLLAMA_MODEL", DEFAULT_OLLAMA_MODEL)

    if not check_ollama_model(effective_model):
        if not prompt_ollama_pull(effective_model):
            return

    print(f"\nStarting Handler server on {host}:{port}\n")
    logger.info("Initializing A2A server with push notification support...")

    effective_api_key = None
    if require_auth:
        effective_api_key = (
            api_key or os.getenv("HANDLER_API_KEY") or generate_api_key()
        )
        print(
            f"Authentication required!\n"
            f"API Key: {effective_api_key}\n"
            f"\nUse with Handler CLI:\n"
            f'  handler message send http://localhost:{port} "message" '
            f"--api-key {effective_api_key}\n"
        )

    agent = create_llm_agent(model=effective_model)
    agent_card = build_agent_card(agent, host, port, require_auth=require_auth)

    streaming_enabled = (
        agent_card.capabilities.streaming if agent_card.capabilities else False
    )
    push_notifications_enabled = (
        agent_card.capabilities.push_notifications if agent_card.capabilities else False
    )
    auth_enabled = agent_card.security_schemes is not None

    logger.info(
        "Agent card capabilities: streaming=%s, push_notifications=%s, auth=%s",
        streaming_enabled,
        push_notifications_enabled,
        auth_enabled,
    )

    a2a_application = create_a2a_application(agent, agent_card, effective_api_key)

    config = uvicorn.Config(a2a_application, host=host, port=port)
    server = uvicorn.Server(config)

    asyncio.run(server.serve())



"""Authentication support for A2A protocol.

Handles credential storage and HTTP authentication header generation.
Currently supports API key and HTTP bearer authentication schemes.
"""

from dataclasses import dataclass
from enum import Enum


class AuthType(str, Enum):
    """Supported authentication types."""

    API_KEY = "api_key"
    BEARER = "bearer"


@dataclass
class AuthCredentials:
    """Authentication credentials for an agent.

    Stores the credential value and metadata about how to apply it.
    """

    auth_type: AuthType
    value: str
    header_name: str | None = None  # For API key: custom header name

    def to_headers(self) -> dict[str, str]:
        """Generate HTTP headers for this credential.

        Returns:
            Dictionary of headers to include in requests
        """
        if self.auth_type == AuthType.BEARER:
            return {"Authorization": f"Bearer {self.value}"}
        elif self.auth_type == AuthType.API_KEY:
            header = self.header_name or "X-API-Key"
            return {header: self.value}
        return {}

    def to_dict(self) -> dict[str, str | None]:
        """Serialize credentials for storage."""
        return {
            "auth_type": self.auth_type.value,
            "value": self.value,
            "header_name": self.header_name,
        }

    @classmethod
    def from_dict(cls, data: dict[str, str | None]) -> "AuthCredentials":
        """Deserialize credentials from storage."""
        return cls(
            auth_type=AuthType(data["auth_type"]),
            value=data.get("value") or "",
            header_name=data.get("header_name"),
        )


def create_bearer_auth(token: str) -> AuthCredentials:
    """Create bearer token authentication."""
    return AuthCredentials(auth_type=AuthType.BEARER, value=token)


def create_api_key_auth(
    key: str,
    header_name: str = "X-API-Key",
) -> AuthCredentials:
    """Create API key authentication.

    Args:
        key: The API key value
        header_name: Header name to use (default: X-API-Key)
    """
    return AuthCredentials(
        auth_type=AuthType.API_KEY,
        value=key,
        header_name=header_name,
    )



"""MCP server CLI command."""

from typing import Literal

import rich_click as click

from a2a_handler.common import get_logger

log = get_logger(__name__)

TransportType = Literal["stdio", "sse", "streamable-http"]


@click.command()
@click.option(
    "--transport",
    "-t",
    type=click.Choice(["stdio", "sse", "streamable-http"]),
    default="stdio",
    help="Transport protocol to use",
    show_default=True,
)
def mcp(transport: TransportType) -> None:
    """Run a local MCP server exposing A2A capabilities.

    This starts an MCP (Model Context Protocol) server that exposes Handler's
    A2A functionality as MCP tools and resources. You can connect to this
    server from any MCP-compatible client (like Claude Desktop, Cursor, etc.).

    The server provides tools for:

    \b
    Card Tools:
    - validate_agent_card: Validate an agent card from URL or file
    - get_agent_card: Retrieve an agent's full card details

    \b
    Message Tools:
    - send_message: Send messages to agents with session/auth support

    \b
    Task Tools:
    - get_task: Get task status and details
    - cancel_task: Cancel a running task
    - set_task_notification: Configure push notification webhooks
    - get_task_notification: Get push notification config

    \b
    Session Tools:
    - list_sessions: List all saved sessions
    - get_session_info: Get session for a specific agent
    - clear_session_data: Clear saved session state

    \b
    Auth Tools:
    - set_agent_credentials: Save bearer token or API key
    - clear_agent_credentials: Remove saved credentials

    Example configuration for Claude Desktop (claude_desktop_config.json):

        {
          "mcpServers": {
            "handler": {
              "command": "handler",
              "args": ["mcp"]
            }
          }
        }
    """
    from a2a_handler.mcp import run_mcp_server

    log.info("Starting MCP server with %s transport", transport)
    run_mcp_server(transport=transport)



"""Handler - A2A protocol client and TUI for agent interaction.

Provides CLI and TUI interfaces for communicating with A2A protocol agents.
"""

from importlib.metadata import version

from a2a_handler.tui import HandlerTUI, main

__version__ = version("a2a-handler")

__all__ = ["__version__", "HandlerTUI", "main"]



"""Session state management for the Handler CLI.

Persists context_id, task_id, and authentication credentials
across CLI invocations for conversation continuity.
"""

import json
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any

from a2a_handler.auth import AuthCredentials
from a2a_handler.common import get_logger

logger = get_logger(__name__)

DEFAULT_SESSION_DIRECTORY = Path.home() / ".handler"
SESSION_FILENAME = "sessions.json"


@dataclass
class AgentSession:
    """Session state for a single agent."""

    agent_url: str
    context_id: str | None = None
    task_id: str | None = None
    credentials: AuthCredentials | None = None

    def update(
        self,
        context_id: str | None = None,
        task_id: str | None = None,
        credentials: AuthCredentials | None = None,
    ) -> None:
        """Update session with new values (only if provided)."""
        if context_id is not None:
            self.context_id = context_id
        if task_id is not None:
            self.task_id = task_id
        if credentials is not None:
            self.credentials = credentials

    def clear_credentials(self) -> None:
        """Clear stored credentials."""
        self.credentials = None


@dataclass
class SessionStore:
    """Persistent store for agent sessions."""

    sessions: dict[str, AgentSession] = field(default_factory=dict)
    session_directory: Path = field(default_factory=lambda: DEFAULT_SESSION_DIRECTORY)

    @property
    def session_file_path(self) -> Path:
        """Path to the session file."""
        return self.session_directory / SESSION_FILENAME

    def _ensure_directory_exists(self) -> None:
        """Ensure the session directory exists."""
        self.session_directory.mkdir(parents=True, exist_ok=True)

    def load(self) -> None:
        """Load sessions from disk."""
        if not self.session_file_path.exists():
            logger.debug("No session file found at %s", self.session_file_path)
            return

        try:
            with open(self.session_file_path) as session_file:
                session_data = json.load(session_file)

            for agent_url, agent_session_data in session_data.items():
                credentials = None
                if cred_data := agent_session_data.get("credentials"):
                    credentials = AuthCredentials.from_dict(cred_data)

                self.sessions[agent_url] = AgentSession(
                    agent_url=agent_url,
                    context_id=agent_session_data.get("context_id"),
                    task_id=agent_session_data.get("task_id"),
                    credentials=credentials,
                )

            logger.debug(
                "Loaded %d sessions from %s",
                len(self.sessions),
                self.session_file_path,
            )

        except json.JSONDecodeError as error:
            logger.warning("Failed to parse session file: %s", error)
        except OSError as error:
            logger.warning("Failed to read session file: %s", error)

    def save(self) -> None:
        """Save sessions to disk."""
        self._ensure_directory_exists()

        session_data: dict[str, Any] = {}
        for agent_url, agent_session in self.sessions.items():
            data: dict[str, Any] = {
                "context_id": agent_session.context_id,
                "task_id": agent_session.task_id,
            }
            if agent_session.credentials:
                data["credentials"] = agent_session.credentials.to_dict()
            session_data[agent_url] = data

        try:
            with open(self.session_file_path, "w") as session_file:
                json.dump(session_data, session_file, indent=2)
            logger.debug(
                "Saved %d sessions to %s",
                len(self.sessions),
                self.session_file_path,
            )
        except OSError as error:
            logger.warning("Failed to write session file: %s", error)

    def get(self, agent_url: str) -> AgentSession:
        """Get or create a session for an agent URL."""
        if agent_url not in self.sessions:
            self.sessions[agent_url] = AgentSession(agent_url=agent_url)
            logger.debug("Created new session for %s", agent_url)
        return self.sessions[agent_url]

    def update(
        self,
        agent_url: str,
        context_id: str | None = None,
        task_id: str | None = None,
        credentials: AuthCredentials | None = None,
    ) -> AgentSession:
        """Update session for an agent and save."""
        agent_session = self.get(agent_url)
        agent_session.update(context_id, task_id, credentials)
        self.save()
        logger.debug(
            "Updated session for %s: context_id=%s, task_id=%s, has_credentials=%s",
            agent_url,
            context_id,
            task_id,
            credentials is not None,
        )
        return agent_session

    def set_credentials(
        self,
        agent_url: str,
        credentials: AuthCredentials,
    ) -> AgentSession:
        """Set credentials for an agent."""
        agent_session = self.get(agent_url)
        agent_session.credentials = credentials
        self.save()
        logger.info("Set credentials for %s", agent_url)
        return agent_session

    def clear_credentials(self, agent_url: str) -> None:
        """Clear credentials for an agent."""
        if agent_url in self.sessions:
            self.sessions[agent_url].clear_credentials()
            self.save()
            logger.info("Cleared credentials for %s", agent_url)

    def get_credentials(self, agent_url: str) -> AuthCredentials | None:
        """Get credentials for an agent."""
        if agent_url in self.sessions:
            return self.sessions[agent_url].credentials
        return None

    def clear(self, agent_url: str | None = None) -> None:
        """Clear session(s).

        Args:
            agent_url: If provided, clear only that agent's session.
                      Otherwise, clear all sessions.
        """
        if agent_url:
            if agent_url in self.sessions:
                del self.sessions[agent_url]
                logger.info("Cleared session for %s", agent_url)
        else:
            session_count = len(self.sessions)
            self.sessions.clear()
            logger.info("Cleared all %d sessions", session_count)
        self.save()

    def list_all(self) -> list[AgentSession]:
        """List all sessions."""
        return list(self.sessions.values())


_global_session_store: SessionStore | None = None


def get_session_store() -> SessionStore:
    """Get the global session store (singleton)."""
    global _global_session_store
    if _global_session_store is None:
        _global_session_store = SessionStore()
        _global_session_store.load()
        logger.debug("Initialized global session store")
    return _global_session_store


def get_session(agent_url: str) -> AgentSession:
    """Get session for an agent URL."""
    return get_session_store().get(agent_url)


def update_session(
    agent_url: str,
    context_id: str | None = None,
    task_id: str | None = None,
) -> AgentSession:
    """Update and persist session for an agent."""
    return get_session_store().update(agent_url, context_id, task_id)


def clear_session(agent_url: str | None = None) -> None:
    """Clear session(s)."""
    get_session_store().clear(agent_url)


def set_credentials(agent_url: str, credentials: AuthCredentials) -> AgentSession:
    """Set credentials for an agent."""
    return get_session_store().set_credentials(agent_url, credentials)


def clear_credentials(agent_url: str) -> None:
    """Clear credentials for an agent."""
    get_session_store().clear_credentials(agent_url)


def get_credentials(agent_url: str) -> AuthCredentials | None:
    """Get credentials for an agent."""
    return get_session_store().get_credentials(agent_url)



"""Rich-click configuration for the CLI."""

import rich_click as click

click.rich_click.TEXT_MARKUP = "markdown"
click.rich_click.SHOW_ARGUMENTS = True
click.rich_click.GROUP_ARGUMENTS_OPTIONS = True
click.rich_click.STYLE_HELPTEXT = ""
click.rich_click.STYLE_OPTION = "cyan"
click.rich_click.STYLE_ARGUMENT = "cyan"
click.rich_click.STYLE_COMMAND = "green"
click.rich_click.STYLE_SWITCH = "bold green"

click.rich_click.OPTION_GROUPS = {
    "handler": [
        {"name": "Global Options", "options": ["--verbose", "--debug", "--help"]},
    ],
    "handler message send": [
        {
            "name": "Message Options",
            "options": ["--stream", "--continue", "--context-id", "--task-id"],
        },
        {
            "name": "Authentication Options",
            "options": ["--bearer", "--api-key"],
        },
        {
            "name": "Push Notification Options",
            "options": ["--push-url", "--push-token"],
        },
    ],
    "handler message stream": [
        {
            "name": "Conversation Options",
            "options": ["--continue", "--context-id", "--task-id"],
        },
        {
            "name": "Authentication Options",
            "options": ["--bearer", "--api-key"],
        },
        {
            "name": "Push Notification Options",
            "options": ["--push-url", "--push-token"],
        },
    ],
    "handler task get": [
        {"name": "Query Options", "options": ["--history-length"]},
    ],
    "handler task notification set": [
        {"name": "Notification Options", "options": ["--url", "--token"]},
    ],
    "handler card get": [
        {"name": "Card Options", "options": ["--authenticated"]},
    ],
    "handler server agent": [
        {"name": "Server Options", "options": ["--host", "--port", "--help"]},
    ],
    "handler server push": [
        {"name": "Server Options", "options": ["--host", "--port", "--help"]},
    ],
    "handler session clear": [
        {"name": "Clear Options", "options": ["--all", "--help"]},
    ],
    "handler auth set": [
        {
            "name": "Auth Type",
            "options": ["--bearer", "--api-key", "--api-key-header"],
        },
    ],
}

click.rich_click.COMMAND_GROUPS = {
    "handler": [
        {"name": "Agent Communication", "commands": ["message", "task"]},
        {"name": "Agent Discovery", "commands": ["card"]},
        {"name": "Authentication", "commands": ["auth"]},
        {"name": "Interfaces", "commands": ["tui", "web", "server"]},
        {"name": "Utilities", "commands": ["session", "version"]},
    ],
    "handler message": [
        {"name": "Message Commands", "commands": ["send", "stream"]},
    ],
    "handler task": [
        {"name": "Task Commands", "commands": ["get", "cancel", "resubscribe"]},
        {"name": "Push Notifications", "commands": ["notification"]},
    ],
    "handler task notification": [
        {"name": "Notification Commands", "commands": ["set"]},
    ],
    "handler card": [
        {"name": "Card Commands", "commands": ["get", "validate"]},
    ],
    "handler server": [
        {"name": "Server Commands", "commands": ["agent", "push"]},
    ],
    "handler session": [
        {"name": "Session Commands", "commands": ["list", "show", "clear"]},
    ],
    "handler auth": [
        {"name": "Auth Commands", "commands": ["set", "show", "clear"]},
    ],
}



"""Card commands for agent card operations."""

import asyncio
import json
from typing import Any

import rich_click as click
from a2a.types import AgentCard

from a2a_handler.common import Output, get_logger
from a2a_handler.service import A2AService
from a2a_handler.validation import (
    ValidationResult,
    validate_agent_card_from_file,
    validate_agent_card_from_url,
)

from ._helpers import build_http_client, handle_client_error

log = get_logger(__name__)


@click.group()
def card() -> None:
    """Agent card operations."""
    pass


@card.command("get")
@click.argument("agent_url")
@click.option(
    "--authenticated", "-a", is_flag=True, help="Request authenticated extended card"
)
def card_get(agent_url: str, authenticated: bool) -> None:
    """Retrieve an agent's card."""
    log.info("Fetching agent card from %s", agent_url)

    async def do_get() -> None:
        output = Output()
        try:
            async with build_http_client() as http_client:
                service = A2AService(http_client, agent_url)
                card_data = await service.get_card()
                log.info("Retrieved card for agent: %s", card_data.name)

                _format_agent_card(card_data, output)

        except Exception as e:
            handle_client_error(e, agent_url, output)
            raise click.Abort()

    asyncio.run(do_get())


def _format_agent_card(card_data: object, output: Output) -> None:
    """Format and display an agent card as JSON."""
    card_dict: dict[str, Any]
    if isinstance(card_data, AgentCard):
        card_dict = card_data.model_dump(exclude_none=True)
    else:
        card_dict = {}
    output.line(json.dumps(card_dict, indent=2))


@card.command("validate")
@click.argument("source")
def card_validate(source: str) -> None:
    """Validate an agent card from URL or file."""
    log.info("Validating agent card from %s", source)
    is_url = source.startswith(("http://", "https://"))

    async def do_validate() -> None:
        output = Output()
        if is_url:
            async with build_http_client() as http_client:
                result = await validate_agent_card_from_url(source, http_client)
        else:
            result = validate_agent_card_from_file(source)

        _format_validation_result(result, output)

        if not result.valid:
            raise SystemExit(1)

    asyncio.run(do_validate())


def _format_validation_result(result: ValidationResult, output: Output) -> None:
    """Format and display validation result."""
    if result.valid:
        output.success("Valid Agent Card")
        output.field("Agent", result.agent_name)
        output.field("Protocol Version", result.protocol_version)
        output.field("Source", result.source)
    else:
        output.error("Invalid Agent Card")
        output.field("Source", result.source)
        output.blank()
        output.line(f"Errors ({len(result.issues)}):")
        for issue in result.issues:
            output.list_item(f"{issue.field_name}: {issue.message}", bullet="✗")



"""Message commands for sending messages to A2A agents."""

import asyncio
from typing import Optional

import rich_click as click

from a2a_handler.auth import AuthCredentials, create_api_key_auth, create_bearer_auth
from a2a_handler.common import Output, get_logger
from a2a_handler.service import A2AService, SendResult
from a2a_handler.session import get_credentials, get_session, update_session

from ._helpers import build_http_client, handle_client_error

log = get_logger(__name__)


@click.group()
def message() -> None:
    """Send messages to A2A agents."""
    pass


@message.command("send")
@click.argument("agent_url")
@click.argument("text")
@click.option("--stream", "-s", is_flag=True, help="Stream responses in real-time")
@click.option("--context-id", help="Context ID for conversation continuity")
@click.option("--task-id", help="Task ID to continue")
@click.option(
    "--continue", "-C", "use_session", is_flag=True, help="Continue from saved session"
)
@click.option("--push-url", help="Webhook URL for push notifications")
@click.option("--push-token", help="Authentication token for push notifications")
@click.option("--bearer", "-b", "bearer_token", help="Bearer token (overrides saved)")
@click.option("--api-key", "-k", help="API key (overrides saved)")
def message_send(
    agent_url: str,
    text: str,
    stream: bool,
    context_id: Optional[str],
    task_id: Optional[str],
    use_session: bool,
    push_url: Optional[str],
    push_token: Optional[str],
    bearer_token: Optional[str],
    api_key: Optional[str],
) -> None:
    """Send a message to an agent and receive a response."""
    log.info("Sending message to %s", agent_url)

    if use_session and not context_id:
        session = get_session(agent_url)
        if session.context_id:
            context_id = session.context_id
            log.info("Using saved context: %s", context_id)

    credentials: AuthCredentials | None = None
    if bearer_token:
        credentials = create_bearer_auth(bearer_token)
    elif api_key:
        credentials = create_api_key_auth(api_key)
    else:
        credentials = get_credentials(agent_url)

    async def do_send() -> None:
        output = Output()
        try:
            async with build_http_client() as http_client:
                service = A2AService(
                    http_client,
                    agent_url,
                    enable_streaming=stream,
                    push_notification_url=push_url,
                    push_notification_token=push_token,
                    credentials=credentials,
                )

                output.dim(f"Sending to {agent_url}...")

                if stream:
                    await _stream_message(
                        service, text, context_id, task_id, agent_url, output
                    )
                else:
                    result = await service.send(text, context_id, task_id)
                    update_session(agent_url, result.context_id, result.task_id)
                    _format_send_result(result, output)

        except Exception as e:
            handle_client_error(e, agent_url, output)
            raise click.Abort()

    asyncio.run(do_send())


@message.command("stream")
@click.argument("agent_url")
@click.argument("text")
@click.option("--context-id", help="Context ID for conversation continuity")
@click.option("--task-id", help="Task ID to continue")
@click.option(
    "--continue", "-C", "use_session", is_flag=True, help="Continue from saved session"
)
@click.option("--push-url", help="Webhook URL for push notifications")
@click.option("--push-token", help="Authentication token for push notifications")
@click.option("--bearer", "-b", "bearer_token", help="Bearer token (overrides saved)")
@click.option("--api-key", "-k", help="API key (overrides saved)")
@click.pass_context
def message_stream(
    ctx: click.Context,
    agent_url: str,
    text: str,
    context_id: Optional[str],
    task_id: Optional[str],
    use_session: bool,
    push_url: Optional[str],
    push_token: Optional[str],
    bearer_token: Optional[str],
    api_key: Optional[str],
) -> None:
    """Send a message and stream the response in real-time."""
    ctx.invoke(
        message_send,
        agent_url=agent_url,
        text=text,
        stream=True,
        context_id=context_id,
        task_id=task_id,
        use_session=use_session,
        push_url=push_url,
        push_token=push_token,
        bearer_token=bearer_token,
        api_key=api_key,
    )


async def _stream_message(
    service: A2AService,
    text: str,
    context_id: Optional[str],
    task_id: Optional[str],
    agent_url: str,
    output: Output,
) -> None:
    """Stream a message and handle events."""
    collected_text: list[str] = []
    last_context_id: str | None = None
    last_task_id: str | None = None
    last_state = None

    async for event in service.stream(text, context_id, task_id):
        last_context_id = event.context_id or last_context_id
        last_task_id = event.task_id or last_task_id
        last_state = event.state or last_state

        if event.text and event.text not in collected_text:
            output.line(event.text)
            collected_text.append(event.text)

    update_session(agent_url, last_context_id, last_task_id)

    output.blank()
    if last_context_id:
        output.field("Context ID", last_context_id, dim_value=True)
    if last_task_id:
        output.field("Task ID", last_task_id, dim_value=True)
    if last_state:
        output.state("State", last_state.value)

    if last_state and last_state.value == "auth-required":
        output.blank()
        output.warning("Authentication required")
        output.line("The agent requires authentication to complete this task.")
        output.line(
            "Set credentials with: handler auth set <agent_url> --bearer <token>"
        )


def _format_send_result(result: SendResult, output: Output) -> None:
    """Format and display a send result."""
    output.blank()
    if result.context_id:
        output.field("Context ID", result.context_id, dim_value=True)
    if result.task_id:
        output.field("Task ID", result.task_id, dim_value=True)
    if result.state:
        output.state("State", result.state.value)

    output.blank()
    if result.needs_auth:
        output.warning("Authentication required")
        output.line("The agent requires authentication to complete this task.")
        output.line(
            "Set credentials with: handler auth set <agent_url> --bearer <token>"
        )
        output.line(
            "Or provide inline: handler message send <agent_url> --bearer <token> ..."
        )
    elif result.text:
        output.markdown(result.text)
    else:
        output.dim("No text content in response")
